(* :Title: Isomers *)

(* :Author: Markus van Almsick *)

(* :Summary:  This package provides functions that render the number of 
isomers, enantiomers, achiral isomers, and diastereoisomers of 
chemical molecules. *)

(* :Context: Isomers`Isomers` *)

(* :Package Version: 2.2.1 *)

(* :Copyright: Copyright 2001, Markus van Almsick *)
(* :History:  Version 1.0, September 1998 *)
(* :History:  Version 1.1, January 1999 *)
(* :History:  Version 1.2, March 1999 *)
(* :History:  Version 1.3, March 1999 *)
(* :History:  Version 1.4, March 1999 *)
(* :History:  Version 2.0, March 1999 *)
(* :History:  Version 2.1, October 1999 *)
(* :History:  Version 2.2, February 2001 *)
(* :History:  Version 2.2.1, June 2002 *)
(* :Keywords: Isomers, Enantiomers, Diastomers, Diastereoisomers, Achiral, 
Chemistry, Molecule, Polya, Groups, Group Theory, Combinatoric *)
(* :Source: see article by  Prof.Dr. Hoenig, Dr. Dolhaine, van Almsick *)
(* :Warning: Only tested on PCs and Macs *)

(* :Mathematica Version: 3.0 / 4.0 *)

(* :Limitation: Valid for achiral substituents only! Will not work with \
	remote kernel ! *)
(* :Discussion: *)

BeginPackage["Isomers`Isomers`"]

Unprotect[
  ParentCompounds,
  ParentCompoundQ,
  ChiralParentCompoundQ,
  DefineParentCompound,
  DeleteParentCompound,
  GetParentCompounds,
  SaveParentCompounds,
  DefaultIsomersDatabase,
  IsomersControlPanel,
  IsomersDatabasePanel,
  ParentCompoundsInDataBase,
  NumberOfSubstituentSites,
  PointGroupGenerators,
  PointGroupElement,
  PointGroupElementQ,
  GeneratePointGroupElements,
  PointGroupCycleStructure,
  PointGroupCycle,
  PointGroup,
  PointGroupReport,
  CycleIndex,
  CycleIndices,
  PolyaPolynomial,
  PolyaVariable,
  NumberOfIsomers,
  NumberOfEnantiomericPairs,
  NumberOfAchiralIsomers,
  NumberOfDiastereoisomers,
  Conformations,Reflections,
  SubstituentsMultiplicity,
  Partitions,
  LimitedPartitions,
  InversePermutation,
  FromCycles,ToCycles,CyclesQ,
  PermutationQ
];

ParentCompounds::usage=
  "ParentCompounds is a list of chemical molecule symbols representing 
skeletons of substitutional isomers. The point group symmetries and 
conformational properties of these parent compounds are stored in 
Isomers`IsomersData.m and other databases. (see also 
?DefineParentCompound and ?GetParentCompounds).";

ParentCompoundQ::usage=
  "ParentCompoundQ[molsymb] is a boolean function that verifies if the 
parent compound represented by molsymb can be found in one of the active 
databases such as Isomers`IsomersData.m. (see also ?DefineParentCompound 
and ?GetParentCompounds).";

ChiralParentCompoundQ::usage=
  "ChiralParentCompoundQ[molsymb] is a boolean function that verifies if the 
parent compound represented by molsymb can be found in one of the active 
databases such as Isomers`IsomersData.m (see also ?DefineParentCompound 
and ?GetParentCompounds), and if the parent compound is chiral.";

DefineParentCompound::usage=
  "DefineParentCompound[molsymb,{generators}] provides a mechanism to extend 
the database of parent compounds. The command installs a new molecule 
symbol molsymb and a new set of point group generators. The point group 
generators must comply with the PointGroupElement data structure (see 
?PointGroupElement). (see also ?DeleteParentCompound).";

DeleteParentCompound::usage=
  "DeleteParentCompound[molsymb] provides a mechanism to remove 
molecule molsymb from the database of parent compounds.";

GetParentCompounds::usage=
  "GetParentCompounds[filename] provides a mechanism to load a 
database of parent compounds. One loads the default database 
Isomers`IsomersData.m, if one omits the filename.";

SaveParentCompounds::usage=
  "SaveParentCompounds[filename] provides a mechanism to save the 
database of parent compounds in a file. One saves the database in 
the default file Isomers`IsomersData.m, if one omits the filename."

DefaultIsomersDatabase::usage=
  "DefaultIsomersDatabase is a variable that stores the path and file
name of the default isomer database, usually Isomers`IsomersData.m."

IsomersControlPanel::usage=
  "IsomersControlPanel is a variable that stores the path and file
name of the isomer control panel, usually Isomers`IsomersControlPanel.nb."

IsomersDatabasePanel::usage=
  "IsomersDatabasePanel is a variable that stores the path and file
name of the isomer control panel, usually Isomers`IsomersDatabasePanel.nb."

IsomersInputPanel::usage=
  "IsomersInputPanel is a variable that stores the path and file
name of the isomer input panel, usually Isomers`IsomersInputPanel.nb."

ParentCompoundsInDataBase::usage=
  "ParentCompoundsInDataBase is a variable that contains all parent 
compound symbols stored in the opened database file."

NumberOfSubstituentSites::usage=
  "NumberOfSubstituentSite[molsymb] returns the number of possible 
substituent sites on the given parent compound molsymb.";
		
PointGroupGenerators::usage=
  "PointGroupGenerators[molsymb] returns a list of PointGroupElements that 
generate all point group transformations, under which the given parent 
compound molsymb is invariant.";
		
PointGroupElement::usage=
  "PointGroupElement[p,r,c] is a data structure that represents a 
point group element. The first argument is a substituent site 
permutation. Such a permutation is a list of positive integers 
depicting the rearrangement of substituent sites due to symmetry 
transformations. The second argument is a boolean flag. r indicates, 
if the point group element contains a reflection/inversion or a 
rotation only. The third argument is a boolean flag c that 
indicates, if a conformational transformation is taken into account. A 
conformational transformation is not a true point group transformation. 
It rotates or twists the bonds of a parent compound.";

PointGroupElementQ::usage=
  "PointGroupElementQ[expr] is a boolean functions that returns 
True if the provided expression expr is of type PointGroupElement. 
Otherwise it returns False.";

GeneratePointGroupElements::usage=
  "GeneratePointGroupElements[{generators}] or 
GeneratePointGroupElements[molsymb] creates all point group 
elements out of a list of generators of type PointGroupElement. 
The list of generators may be provided via a parent compound 
symbol molsymb, if the corresponding generator set is implemented 
in the ParentCompounds-database.";

PointGroupCycleStructure::usage=
  "PointGroupCycleStructure[molsymb] renders all point group cycles 
and their element types of parent compound molsymb. The boolean option 
\"Conformations\" may be used to include or exclude conformational 
transformations.";

PointGroupCycle::usage=
  "PointGroupCycle[{p,c},r] is a data structure representing a point 
group element at position p in a group cycle of length c.
r is a boolean flag indicating if the point group element contains
a reflections.";

PointGroup::usage=
  "PointGroup[molsymb] renders a list of possible point groups 
for the parent compound molsymb. The boolean option \"Conformations\" 
may be used to include or exclude possible conformational 
transformations.";

PointGroupReport::usage=
  "PointGroupReport[molsymb] renders the cycle structure and a list 
of possible point groups for parent compound molsymb. \n
PointGroupReport is also a boolean option for the 
DefinePointGroupGenerators command in the Isomers`Isomers`-package. 
By setting PointGroupReport to True, one obtains
a point group identification report. The default setting is True.";

PolyaPolynomial::usage=
  "PolyaPolynomial[molsymb,{vars}] renders the Polya polynomial for 
parent compound molsymb and the given list of variables {vars}. 
The Polya polynomial is a generating functions for the numbers of 
isomers. The variables represent the substituents, the coefficients 
the number of corresponding isomers. ";

CycleIndex::usage=
  "CycleIndex[molsymb,opts] renders the Polya cycle index of parent 
compound molsymb. Options \"Reflections\" and \"Conformations\" may 
be used to include or exclude the respectived symmetry transformations.";

CycleIndices::usage=
  "CycleIndices[molsymb] is a matrix of Polya cycle indices. It 
encodes the Polya cycle indices for point groups with and without 
reflections, and with and without conformational transformations."
  
PolyaVariable::usage=
  "PolyaVariable[b,p] is a construct used in Polya cycle indices."
  
NumberOfIsomers::usage=
  "NumberOfIsomers[molsymb,s,n] calculates the number of isomers that 
one can form by attaching s substituents of n different types to the 
parent compound molsymb. The first argument defines the molecule by its 
symbolic name. The molecule should be part of the molecule database 
(see ?ParentCompounds). If it is not included, one can provide the 
necessary structual information with the DefineParentCompound command. 
Note, that when dealing with a chiral parent compound, both, the original 
sceleton and its mirror image are considered in the isomer enumeration. 
The second argument s denotes the number of occupied substituent sites. 
One can define the number of occupied sites with a single positive 
integer. If one wants to differentiate between different types of 
substituents, one can insert a list of positive integers 
{s1,s2,s3,...}. Then,integer s1 stands for the number of sites occupied 
by substituents of type 1. s2 is the number of sites with substituents 
of type 2, and so on. The total number of occupied sites is 
s = s1+s2+... . The third argument n defines the number of different 
substituent types available. Number n can be greater or equal the 
number of different substituent types actually attached to the molecule. 
NumberOfIsomers then renders the number of isomers with substituents 
taken from an unlimited pool of n different substituents types.";

Reflections::usage=
  "Reflections is a boolean option for commands in the 
Isomers`Isomers`-package. By setting Reflections to True, one 
includes all reflections or inversions (parity transformations) in the 
set of point group symmetry transformations. The default is False. 
Hence, mirror transformations are usually not considered.";

Conformations::usage=
  "Conformations is a boolean option for commands in the 
Isomers`Isomers`-package. By setting Conformations to True, one 
includes conformational transformations (twisting or rotating bonds) 
in the set of (non rigid) point group symmetry transformations. 
The default is True.";

NumberOfEnantiomericPairs::usage=
  "NumberOfEnantiomericPairs[molsymb,s,n] calculates the number of 
enantiomer pairs that one can form by attaching s substituents of n 
different types to the parent compound molsymb. The first argument defines 
the molecule by its symbolic name. The molecule should be part of the 
molecule database (see ?ParentCompounds). If it is not included, 
one can provide the necessary structual information with the 
DefineParentCompound command. Note, that when dealing with a chiral parent 
compounds, both, the original sceleton and its mirror image, are considered 
in the isomer enumeration. The second argument s denotes the number 
of occupied substituent sites. One can define the number of occupied 
sites with a single positive integer. If one wants to differentiate 
between different types of substituents, one can insert a list of 
positive integers {s1,s2,s3,...}. Then,integer s1 stands for the 
number of sites occupied by substituents of type 1. s2 is the number of 
sites with substituents of type 2, and so on. The total number of 
occupied sites is s = s1+s2+... . The third argument n defines the 
number of different substituent types available. 
Number n can be greater or equal the number of different substituent 
types actually attached to the molecule. NumberOfEnantiomericPairs then 
renders the number of enantiomer pairs with substituents taken from an 
unlimited pool of n different substituents types.";

NumberOfAchiralIsomers::usage=
  "NumberOfAchiralIsomers[molsymb,s,n] calculates the number of 
achiral isomers that one can form by attaching s substituents of n 
different types to the parent compound molsymb. The first argument defines 
the molecule by its symbolic name. The molecule should be part of 
the molecule database (see ?ParentCompounds). If it is not included, 
one can provide the necessary structual information with the 
DefineParentCompound command. The second argument s denotes the number 
of occupied substituent sites. One can define the number of occupied 
sites with a single positive integer. If one wants to differentiate 
between different types of substituents, one can insert a list of 
positive integers {s1,s2,s3,...}. Then,integer s1 stands for the 
number of sites occupied by substituents of type 1. s2 is the number 
of sites with substituents of type 2, and so on. The total number of 
occupied sites is s = s1+s2+... . The third argument n defines the 
number of different substituent types available. Number n can be 
greater or equal the number of different substituent types 
actually attached to the molecule. NumberOfAchiralIsomers then 
renders the number of achiral isomers with substituents taken from 
an unlimited pool of n different substituents types.";

NumberOfDiastereoisomers::usage=
  "NumberOfDiastereoisomers[molsymb,s,n] calculates the number of 
diastereoisomers that one can form by attaching s substituents of n 
different types to the parent compound molsymb. The first argument defines 
the molecule by its symbolic name. The molecule should be part of the 
molecule database (see ?ParentCompounds). If it is not included, 
one can provide the necessary structual information with the 
DefineParentCompound command. The second argument s denotes the number 
of occupied substituent sites. One can define the number of occupied 
sites with a single positive integer. If one wants to differentiate 
between different types of substituents, one can insert a list of 
positive integers {s1,s2,s3,...}. Then,integer s1 stands 
for the number of sites occupied by substituents of 
type 1. s2 is the number of sites with substituents of type 2, and so on. 
The total number of occupied sites is s = s1+s2+... . The third argument n 
defines the number of different substituent types available. 
Number n can be greater or equal the number of different 
substituent types actually attached to the molecule. 
NumberOfDiastereoisomers then renders the number of achiral 
isomers with substituents taken from an unlimited pool of n 
different substituents types.";

SubstituentsMultiplicity::usage=
  "SubstituentsMultiplicity[{s1,s2,s3,...},n] returns the number of 
possibilities to distribute substituents of n different types into a 
partition {s1,s2,s3,...}. Each slot of the partition may only be filled 
with substituents of the same type. The number of available substituents 
may be left unspecified (symbolic n), but Slot 1 must contain s1 
substituents, slot 2 must contain s2 substituents, and so on.";

Partitions::usage = 
  "Partitions[n] constructs all partitions of integer n in reverse 
lexicographic order. Partitions[n,m] constructs all partitions of integer 
n with all summands being smaller or equal m. Partitions[n,m,l] constructs 
partitions with no more than l parts (or summands).";

LimitedPartitions::usage = 
  "LimitedPartitions[n,{l1,l2,...}] constructs all limited partitions of 
integer n. The limitations are upper limits {l1,l2,...} for each partition 
summand. The upper limits are provided by the integer list in the second 
argument. Note, that limited partitions are not orderless!";

InversePermutation::usage = 
  "InversePermutation[p] renders the inverse permutation of permutation p."
  
FromCycles::usage = 
  "FromCycles[c] renders a permutation in cycle structure c as a 
regular permutation list."

ToCycles::usage = 
  "ToCycles[p] yields the cycle structure of permutation p."
  
PermutationQ::usage =
  "PermutationQ[expr] is a boolean function that verifies if expr is a 
well defined permutation list."

CyclesQ::usage =
  "CyclesQ[expr] is a boolean function that verifies if expr is a well 
defined list of cycles representing a permutation."

System`ShowGroupOpenCloseIcon;System`CellGroupingRules;
System`ButtonBoxOptions;System`Inherited;

Begin["`Private`"]

(* Parent Compounds *)

(* Initials *)

CallUpNotebook = SelectedNotebook[];

NotebookWrite[
CallUpNotebook,
Cell[TextData[{
  StyleBox[" 2000,2001  Markus van Almsick\nThis ",
    CellFrame->True],
  StyleBox["Mathematica",
    CellFrame->True,
    FontSlant->"Italic"],
  StyleBox[" package, version 2.2, is for ",
    CellFrame->True],
  StyleBox["academic use only",
    CellFrame->True,
    FontVariations->{"Underline"->True}],
  StyleBox["!\nPlease, cite \"Efficient Algorithm To Enumerate \
Isomers and Diamutamers with More Than One Type of Substituent\" \
in ",
    CellFrame->True],
  "J. Chem. Inf. Comput. Sci. 2000, 40, 956-966",
  StyleBox[".\nPlease, contact  ",
    CellFrame->True],
  StyleBox[ButtonBox["m.van.almsick@cityweb.de",
    ButtonData:>{
      URL[ "mailto:m.van.almsick@cityweb.de"], None},
    ButtonStyle->"Hyperlink"],
    CellFrame->True],
  StyleBox["  for commercial use. Thank you!",
    CellFrame->True]
}], "Text",
  Editable->False,
  Deletable->False,
  CellFrame->True,
  ShowCellBracket->False,
  ShowSelection->False,
  Background->RGBColor[1, 0.4, 0.4],
  Magnification->1]
];

ParentCompoundQ[s_] := MemberQ[ParentCompounds,s];

ChiralParentCompoundQ[s_] := 
	And[
		ParentCompoundQ[s],
		FreeQ[PointGroupGenerators[s],PointGroupElement[_,True,_]]
	];

ds = If[$MachineType === "PC",$PathnameSeparator,""];

ds = $PathnameSeparator;

(* Initialize List of Parent Compounds *)

If[
	Head[ParentCompounds]=!=List,
	Unprotect[ParentCompounds];
	ParentCompounds={};
	Protect[ParentCompounds]
];


(* Set Path to Isomers Database *)

Isomers::nodata=
  "Cannot find the \"IsomersData.m\" default data base of parent compounds.";

Isomers::manydata=
  "More than one default data base of parent compounds was found. `1` will 
be used.";

With[
	{fm = 
		FileNames[
			{"IsomersData.m","ISOMERSDATA.M","isomersdata.m"},
			Map[StringJoin[#,ds,DirectoryName[$Input]]&,$Path],1
		]
	},
	If[fm === {},
		Message[Isomers::nodata];
		DefaultIsomersDatabase =.,
		If[Length[fm] >= 2 ,
			Message[Isomers::manydata,First[fm]]
		];
		DefaultIsomersDatabase = First[fm]
	]
];


(* Set Path to Isomers Control Panel *)

Isomers::noctrlpanel=
  "Cannot find the \"IsomersControlPanel.nb\" file.";

Isomers::manyctrlpanel=
  "More than one \"IsomersControlPanel.nb\" file was found. `1` will 
be used.";

With[
	{fm = 
		FileNames[
			{"IsomersControlPanel.nb","ISOMERSCONTROLPANEL.NB",
			 "isomerscontrolpanel.nb"},
			Map[StringJoin[#,ds,DirectoryName[$Input]]&,$Path],1
		]
	},
	If[fm === {},
		Message[Isomers::noctrlpanel];
		IsomersControlPanel =.,
		If[Length[fm] >= 2 ,
			Message[Isomers::manyctrlpanel,First[fm]]
		];
		IsomersControlPanel = First[fm];
		icp = NotebookOpen[IsomersControlPanel]
	]
];


(* Set Path to Isomers Database Panel *)

Isomers::nodatapanel=
  "Cannot find the \"IsomersDatabasePanel.nb\" file.";

Isomers::manydatapanel=
  "More than one \"IsomersDatabasePanel.nb\" file was found. `1` will 
be used.";

With[
	{fm = 
		FileNames[
			{"IsomersDatabasePanel.nb","ISOMERSDATABASEPANEL.NB","
isomersdatabasepanel.nb"},
			Map[StringJoin[#,ds,DirectoryName[$Input]]&,$Path],1
		]
	},
	If[fm === {},
		Message[Isomers::nodatapanel];
		IsomersDatabasePanel =.,
		If[Length[fm] >= 2 ,
			Message[Isomers::manydatapanel,First[fm]]
		];
		IsomersDatabasePanel = First[fm];
		idp = NotebookOpen[IsomersDatabasePanel]
	]
];

ParentCompound=.;

(* Set Path to Isomers Input Panel *)

Isomers::noinpanel=
  "Cannot find the \"IsomersInputPanel.nb\" file.";

Isomers::manyinpanel=
  "More than one \"IsomersInputPanel.nb\" file was found. `1` will 
be used.";

With[
	{fm = 
		FileNames[
			{"IsomersInputPanel.nb","ISOMERSINPUTPANEL.NB",
			 "isomersinputpanel.nb"},
			Map[StringJoin[#,ds,DirectoryName[$Input]]&,$Path],1
		]
	},
	If[fm === {},
		Message[Isomers::noinpanel];
		IsomersInputPanel =.,
		If[Length[fm] >= 2 ,
			Message[Isomers::manyinpanel,First[fm]]
		];
		IsomersInputPanel = First[fm]
	]
];
	 
	 
SetSelectedNotebook[CallUpNotebook];
	 

(* Parent Compounds I/O *)

SaveParentCompounds[Global`\[Placeholder]] := SaveParentCompounds[];

SaveParentCompounds[filename_String:DefaultIsomersDatabase] := 
	Block[
		{IsomersDataStream},
		IsomersDataStream=
			OpenWrite[
				filename,
				FormatType -> OutputForm,
				PageWidth -> Infinity
			];
		Print[filename];
		WriteString[
			IsomersDataStream,
      		Apply[StringForm[
            "(* This file was created by the Isomers/Isomers.m-package 
on `2`/`3`/`1` at `4`:`5`:`6`. *)\n",##]&,Date[]]
		];
		Write[
			IsomersDataStream,
      		HoldForm[ParentCompoundsInDataBase=#;]&[InputForm[ParentCompounds]]
      	];
		Scan[
			Write[
				IsomersDataStream,
          		HoldForm[
          			NumberOfSubstituentSites[#1]^=#3;
            		PointGroupGenerators[#1]^=#4;
            		#2/:PointGroupCycleStructure[#1, Conformations -> False]=#5;
            		#2/:PointGroupCycleStructure[#1, Conformations -> True]=#6;
            		CycleIndices[#1]^=#7;
            	]&[
            		InputForm[#],
            		InputForm[If[Depth[#]>1,Head[#],#]],
            		InputForm[NumberOfSubstituentSites[#]],
            		InputForm[PointGroupGenerators[#]],
            		InputForm[PointGroupCycleStructure[#,Conformations->False]],
            		InputForm[PointGroupCycleStructure[#,Conformations->True]],
            		InputForm[CycleIndices[#]]
            	]
          	]&,
      		Sort[ParentCompounds]
      	];
		Close[IsomersDataStream]
	];
	
Isomers::readdata=
  "The data in file `1` appear to be corrupted.";

GetParentCompounds[Global`\[Placeholder]] := GetParentCompounds[];
	
GetParentCompounds[filename_String:DefaultIsomersDatabase] := 
	Block[
		{IsomersDataStream ,ParentCompoundSymbols},
		IsomersDataStream= OpenRead[filename];
		Skip[IsomersDataStream,Expression];
		Unprotect[ParentCompounds,ParentCompoundsInDataBase];
		Check[
		If[Read[filename]=!=$Failed,
			ParentCompounds=Union[ParentCompounds,ParentCompoundsInDataBase];
			Protect[ParentCompounds,ParentCompoundsInDataBase];
			ParentCompoundSymbols=
      			Union[
      				Map[
      					Replace[#,{s_Symbol->s,s_:>Head[s]}]&,
      					ParentCompoundsInDataBase
      				]
      			];
			Scan[Set[MessageName[#,"usage"],
				ToString[#]<>
          " is used as a parent compound symbol representing a chemical 
parent compound in the substitutional isomer enumeration calculations of 
the Isomers`Isomers`-Package."]&,ParentCompoundSymbols];
			Apply[Unprotect,ParentCompoundSymbols];
			While[Read[IsomersDataStream]=!=EndOfFile];
			Apply[Protect,ParentCompoundSymbols]
		],
		Message[Isomers::readdata,filename],Read::readt];
		Close[IsomersDataStream];
		ResetParentCompoundDatabase
	];


(* Define Parent Compound *)

(* Generator Consistency Check *)

DefineParentCompound::nopge=
  "The provided generators do not exhibit the PointGroupElement data 
structure.";

DefineParentCompound::noperm=
  "The provided generators exhibit ill defined permutations.";

DefineParentCompound::difford=
  "The provided generators permute different numbers of elements.";

DefineParentCompound::dblperm=
  "Warning: Two or more generators are the same.";
  
GeneratorConsistencyCheck[gens:{__PointGroupElement}]:=
	Block[
		{perms},
		If[
			Not[Apply[And,
          Map[MatchQ[#,PointGroupElement[_List,False|True,False|True]]&,
            gens]]],
			Message[DefineParentCompound::nopge];Return[]
		];
		perms = Map[First,gens];
		If[
			Not[Apply[And,Map[PermutationQ,perms]]],
			Message[DefineParentCompound::noperm]
		];
		If[
			Not[MatrixQ[perms]],
			Message[DefineParentCompound::difford]
      	];
		If[
			Not[Apply[UnsameQ,gens]],
			Message[DefineParentCompound::dblperm]
		]
	];


(* Install Parent Compound Name *)

SetAttributes[FreeSymbolQ,HoldFirst];

FreeSymbolQ[s:(sh_Symbol | sh_Symbol[__])] :=
  Not[ValueQ[s]];
  
DefineParentCompound::predef=
  "The new input for the parent compound symbol `1` replaces an older \
record in the database.";

DefineParentCompound::badsymb=
  "The proposed parent compound symbol cannot be used. It is protected \
or defined in some other way.";

InstallParentCompoundName[s:(sh_Symbol | sh_Symbol[__])] :=
	If[
		MemberQ[ParentCompounds,s],
		Message[DefineParentCompound::predef,s],
     If[
			FreeSymbolQ[s],
			Unprotect[ParentCompounds];
				AppendTo[ParentCompounds,s];
      		Protect[ParentCompounds],
      	Message[DefineParentCompound::badsymb];
      ]
	];


(* Define Parent Compound *)

DefineParentCompound::nogens=
  "No point group generators were provided. However, at least the 
number of substituent sites must be given.";

Options[DefineParentCompound] = {PointGroupReport->True};

SetAttributes[DefineParentCompound,HoldFirst];

DefineParentCompound[s:(_Symbol | _Symbol[__]), {},opts___Rule] := 
	Message[DefineParentCompound::nogens];

DefineParentCompound[s:(sh_Symbol | sh_Symbol[__]),n_Integer?Positive,
    opts___Rule] :=
	Block[
		{pges},
		Check[InstallParentCompoundName[s],Return[],
			DefineParentCompound::badsymb];
		Unprotect[sh];
		NumberOfSubstituentSites[s] ^= n;
		PointGroupGenerators[s] ^= {PointGroupElement[Range[n],False,False]};
		CycleIndices[s] ^=List[List[PolyaVariable[1,n],0],List[0,0]];
		Protect[sh];
		ResetParentCompoundDatabase;
		SetParentCompoundButton[s];
		pges=GeneratePointGroupElements[s];
		If[
			PointGroupReport /. {opts} /. Options[DefineParentCompound],
			PointGroupReport[pges]
		];
	];

DefineParentCompound[s:(sh_Symbol | sh_Symbol[__]), 
    gens:{__PointGroupElement},opts___Rule] :=
	Block[
		{pges},
		Check[GeneratorConsistencyCheck[gens],Return[],
			DefineParentCompound::nopge,
			DefineParentCompound::noperm,
			DefineParentCompound::difford];
		Check[InstallParentCompoundName[s],Return[],
			DefineParentCompound::badsymb];
		Unprotect[sh];
		NumberOfSubstituentSites[s] ^= Length[First[First[gens]]];
		PointGroupGenerators[s] ^= gens;
		pges=GeneratePointGroupElements[PointGroupGenerators[s]];
		CycleIndices[s] ^=CycleIndices[pges];
		Protect[sh];
		ResetParentCompoundDatabase;
		SetParentCompoundButton[s];
		If[
			PointGroupReport /. {opts} /. Options[DefineParentCompound],
			PointGroupReport[pges]
		];
	];


(* Delete Parent Compound *)

SetAttributes[DeleteSymmetryGenerators,HoldFirst];

DeleteParentCompound[s:(sh_Symbol | sh_Symbol[__])] :=
	(Unprotect[sh];
	Off[TagUnset::norep];
	TagUnset[sh,CycleIndices[s]];
	TagUnset[sh,NumberOfSubstituentSites[s]];
	TagUnset[sh,PointGroupGenerators[s]];
   	On[TagUnset::norep];
	Protect[sh];
	Unprotect[ParentCompounds];
	ParentCompounds=DeleteCases[ParentCompounds,s];
	Protect[ParentCompounds];
	ResetParentCompoundDatabase;);


(* Point Groups *)

(* Permutation Group Element *)

PermutationQ[perm_] := MatchQ[Sort[perm],Range[Length[perm]]];

InversePermutation[perm_?PermutationQ] := 
  Level[Map[Position[perm,#]&, Range[Length[perm]]],{3}];
  

(* Point Group Element *)

PointGroupElement /: 
  MakeBoxes[PointGroupElement[p_List,s:True|False,c:True|False],
StandardForm]:=
	RowBox[{
      If[c||s,
        Which[c&&s,SubscriptBox["\[ScriptCapitalR]","c\[Sigma]"],c,
          SubscriptBox["\[ScriptCapitalR]","c"],s,
          SubscriptBox["\[ScriptCapitalR]","\[Sigma]"]],"\[ScriptCapitalR]"],
		"[",MakeBoxes[p,StandardForm][[1,2]], "]"
    }];
      
MakeExpression[
	RowBox[{"\[ScriptCapitalR]","[", p_,"]"}], 
	StandardForm
] :=
  MakeExpression[
		RowBox[{"PointGroupElement", "[", 
      RowBox[{ 
        RowBox[{"{", 
          p, "}"}]}],",","False", ",", "False", "]"}], 
		StandardForm
	];
	
MakeExpression[
	RowBox[{SubscriptBox["\[ScriptCapitalR]", sc_String],"[", p_,"]"}], 
	StandardForm
] :=
  MakeExpression[
		RowBox[{"PointGroupElement", "[", 
      RowBox[{ 
        RowBox[{"{", 
          p, "}"}]}],",",ToString[StringMatchQ[sc,"*\[Sigma]"]], ",", 
        ToString[StringMatchQ[sc,"c*"]], "]"}], 
		StandardForm
	];
	
MakeExpression[
	RowBox[{"(", 
		GridBox[{
          _List?((MapAll[ToExpression,#]===Range[Length[#]])&),
          p_List?((Sort[MapAll[ToExpression,#]]===Range[Length[#]])&)}
        ],
	")"}], 
	StandardForm
] :=
	MakeExpression[
		RowBox[{"PointGroupElement", "[",ToExpression[p],",", 
				"False", ",", "False","]"}
        ], 
		StandardForm
	];
	
MakeExpression[
	SubscriptBox[RowBox[{"(", 
		GridBox[{
          _List?((MapAll[ToExpression,#]===Range[Length[#]])&),
          p_List?((Sort[MapAll[ToExpression,#]]===Range[Length[#]])&)}
        ],
	")"}], ind_String], 
	StandardForm
] :=
	MakeExpression[
		RowBox[
			{"PointGroupElement", "[", 
			RowBox[{ToExpression[p], ",", 
        			ToString[StringMatchQ[ind,"*\[Sigma]*"]], ",", 
            		ToString[StringMatchQ[ind,"*c*"]]
			}], 
			"]"}
		], 
		StandardForm
	];
	
PointGroupElement/:
	PointGroupElement[p1_List,s1:False|True,c1:False|True] ** 
    PointGroupElement[p2_List,s2:False|True,c2:False|True] :=
	PointGroupElement[Part[p1,p2],Xor[s1,s2],Or[c1,c2]];


(* PointGroupElementQ *)

PointGroupElementQ[PointGroupElement[p_List,False|True,False|True]]:=
	If[Sort[p]==Range[Length[p]],True,False];
	
PointGroupElementQ[_] = False;


(* Generate Point Group Elements *)

GeneratePointGroupElements::overflow = 
  "The provided generators create an unreasonably large group. Sorry, 
I give up!";

GeneratePointGroupElements[gens:{__PointGroupElement}] := 
	MemoryConstrained[FixedPoint[
		Union[Flatten[Outer[NonCommutativeMultiply,#,#,1],1],#,
          SameTest -> (SameQ[Drop[#1,-1],Drop[#2,-1]]&)]&,
		gens
	],10485760,Message[GeneratePointGroupElements::overflow];Abort];
GeneratePointGroupElements[mol_?ParentCompoundQ] := 
	GeneratePointGroupElements[PointGroupGenerators[mol]];


(* Generate Group *)

GeneratePointGroupTable[pges:{__PointGroupElement}] :=
	GroupTable[Sort[#],Map[First,#]]&[
		Map[
			Map[Last, Sort[Transpose[{#, Range[Length[#]]}]]]&,
			Outer[NonCommutativeMultiply,pges,pges,1],
			{1}
		]
	];


(* Extract Subgroup *)

SubPointGroupIndices[pges:{__PointGroupElement}] :=
	Outer[
		Flatten[Position[pges,PointGroupElement[_,#1,#2]]]&,
		{False,True},{False,True}
	];
	
ExtractSubgroupTable[subind:{__Integer?Positive}][
    gti:GroupTable[gt:{{__Integer?Positive}..},inv_?PermutationQ]]:=
	GroupTable[Map[(#[[subind]])&,gt[[subind]]],inv[[subind]]]/.Thread[
      Rule[subind,Range[Length[subind]]]];
      
ConvertToSubgroupIndices[subind:{__Integer?Positive}][{}] = {};
	
ConvertToSubgroupIndices[subind:{__Integer?Positive}][
    ind:{__Integer?Positive}]:=
	Flatten[Map[Position[subind,#]&,ind]];


(* Group Classes *)

GroupClasses[GroupTable[gt:{{__Integer}..},inv_?PermutationQ]] :=
	Block[
		{cls = {},elems = First[gt]},
		While[
			elems != {},
			AppendTo[
				cls, 
				Union[
					MapThread[Part[gt,#1,#2]&,{inv,gt[[First[elems]]]}]
				]
			];
			elems = Complement[elems,Last[cls]]
		];
		Return[cls]
	];


(* Group Cycles *)

GroupCycleOf[{1,___}] = {1};

GroupCycleOf[p_List] :=
	FixedPointList[Part[p,#]&,First[p],SameTest -> (SameQ[1,#2]&)];

GroupCycles[gt:{{__Integer}..}] := 
	Block[
		{cycls = {},elems =First[gt]},
		While[
			elems != {},
			AppendTo[cycls,GroupCycleOf[gt[[First[elems]]]]];
			elems = Complement[elems,Last[cycls]];
		];
		elems = Level[Cases[cycls,{n_,r__,_} :> Select[{r},Less[#,n]&]],{2}];
		DeleteCases[cycls,{Apply[Alternatives,elems],__}]
	];
	
GroupCycles[gt:{{__Integer}..},subind:{1,___Integer?Positive}] := 
	Block[
		{cycls = {},elems =subind},
		While[
			elems != {},
			AppendTo[cycls,GroupCycleOf[gt[[First[elems]]]]];
			elems = Complement[elems,Last[cycls]];
		];
		elems = Level[Cases[cycls,{n_,r__,_} :> Select[{r},Less[#,n]&]],{2}];
		DeleteCases[cycls,{Apply[Alternatives,elems],__}]
	];


(* Group Cycle Index *)

GroupCycleIndex[gt:{{__Integer}..}] :=
	With[
		{cl = Map[Length,cycls =Rest[ GroupCycles[gt]]]},
		Prepend[
			Apply[
				{cl[[#1]],#2}&,
				Map[First[Position[cycls,#]]&,Rest[gt[[1]]]],
				{1}],
			{1,1}
		]
	];
	
GroupCycleIndex[gt:{{__Integer}..},subind:{1,___Integer?Positive}] :=
	With[
		{cl = Map[Length,cycls =Rest[ GroupCycles[gt,subind]]]},
		Prepend[
			Apply[
				{cl[[#1]],#2}&,
				Map[First[Position[cycls,#]]&,Rest[subind]],
				{1}],
			{1,1}
		]
	];


(* Point Group Cycle Structure *)
	
Options[PointGroupCycleStructure] = {Conformations-> False};


PointGroupCycleStructure[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ, 
	opts___Rule
] := 
	PointGroupCycleStructure[s,GeneratePointGroupElements[s],opts];

PointGroupCycleStructure[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ, 
	pges:{__PointGroupElement},
	___Rule,Conformations->False,___Rule
] :=
	Block[
		{gti = GeneratePointGroupTable[pges],
		 sgi = Transpose[SubPointGroupIndices[pges]],
		 pgcs},
		If[	Last[sgi]==={{},{}},
			pgcs=Apply[PointGroupCycleStructure[gti,##]&,First[sgi]],
			pgcs=Apply[
				PointGroupCycleStructure[
					ExtractSubgroupTable[Apply[Join,{##}]][gti],
            		ConvertToSubgroupIndices[Apply[Join,{##}]][#1],
            		ConvertToSubgroupIndices[Apply[Join,{##}]][#2]
            	]&,First[sgi]
            ]
        ];
		Unprotect[sh];
        sh /: PointGroupCycleStructure[s,Conformations->False] = pgcs;
		Protect[sh];
		Return[PointGroupCycleStructure[s,Conformations->False]]
	];

PointGroupCycleStructure[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ, 
	pges:{__PointGroupElement},
	___Rule,Conformations->True,___Rule
] :=
	Block[
		{gti = GeneratePointGroupTable[pges],
		 sgi = Transpose[SubPointGroupIndices[pges]],
		 pgcs},
		pgcs=Apply[PointGroupCycleStructure[gti,##]&,MapThread[Join,sgi]];
		Unprotect[sh];
		sh /: PointGroupCycleStructure[s,Conformations->True] = pgcs;
		Protect[sh];
		Return[PointGroupCycleStructure[s,Conformations->True]]
	];

		 
PointGroupCycleStructure[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ, 
	pges:{__PointGroupElement},
	___Rule
] := 
	Block[
		{gti = GeneratePointGroupTable[pges],
		 sgi = Transpose[SubPointGroupIndices[pges]],
		 pgcs},
		Unprotect[sh];
		If[	Last[sgi]==={{},{}},
			pgcs=Apply[PointGroupCycleStructure[gti,##]&,First[sgi]],
			pgcs=Apply[PointGroupCycleStructure[gti,##]&,MapThread[Join,sgi]]
        ];
		sh /: PointGroupCycleStructure[s,Conformations->True] = pgcs;
		If[	Last[sgi]=!={{},{}},
			pgcs=Apply[
				PointGroupCycleStructure[
					ExtractSubgroupTable[Apply[Join,{##}]][gti],
            		ConvertToSubgroupIndices[Apply[Join,{##}]][#1],
            		ConvertToSubgroupIndices[Apply[Join,{##}]][#2]
            	]&,First[sgi]
            ]
        ];
        sh /: PointGroupCycleStructure[s,Conformations->False] = pgcs;
		Protect[sh];
		Return[{
			PointGroupCycleStructure[s,Conformations->False],
			PointGroupCycleStructure[s,Conformations->True]
		}]
	];

PointGroupCycleStructure[
    gti:GroupTable[gt:{{__Integer}..},inv_?PermutationQ],
    rotind:{1,___Integer?Positive},refind:{___Integer?Positive}
] :=
	With[
		{gci=GroupCycleIndex[gt,Sort[Join[rotind,refind]]]},
		Apply[PointGroupCycleStructure,
      	Sort[Map[Map[(Length[#]*First[#])&,Split[Sort[#]]]&,Map[
			Join[
				Map[PointGroupCycle[#,False]&,gci[[rotind]]],
				Map[PointGroupCycle[#,True]&,gci[[refind]]]
			][[#]]&,
			Map[
				InversePermutation[Join[rotind,refind]][[#]]&,
				GroupClasses[gti]
			]
		],{1}]]]
	];
	
PointGroupCycle /: MakeBoxes[PointGroupCycle[{1,1},False],StandardForm] =
  "\[ScriptCapitalE]";
  
PointGroupCycle /: MakeBoxes[PointGroupCycle[{2,1},True],StandardForm] =
  "\[Sigma]";
  
PointGroupCycle /: 
  MakeBoxes[PointGroupCycle[{n_Integer,1},False],StandardForm] :=
	SubscriptBox["\[ScriptCapitalC]", MakeBoxes[n,StandardForm]];
	
PointGroupCycle /: 
  MakeBoxes[PointGroupCycle[{n_Integer,p_Integer},False],StandardForm] :=
	SubsuperscriptBox["\[ScriptCapitalC]", MakeBoxes[n,StandardForm],
    MakeBoxes[p,StandardForm]];
    
PointGroupCycle /: 
  MakeBoxes[PointGroupCycle[{n_Integer,1},True],StandardForm] :=
	SubscriptBox["\[Sigma]\[ScriptCapitalC]", MakeBoxes[n,StandardForm]];
	
PointGroupCycle /: 
  MakeBoxes[PointGroupCycle[{n_Integer,p_Integer},True],StandardForm] :=
	SubsuperscriptBox["\[Sigma]\[ScriptCapitalC]", MakeBoxes[n,StandardForm],
    MakeBoxes[p,StandardForm]];
    
PointGroupCycleStructure /: 
  MakeBoxes[PointGroupCycleStructure[p__List],StandardForm] :=
	MakeSequenceBox[
		Map[DisplayForm[MakeSequenceBox[#,Bracket -> Round]]&,{p}],
		Bracket -> Round
	];
	
MakeSequenceBox[{}] = RowBox[{}];

MakeSequenceBox[{expr_}] := MakeBoxes[expr,StandardForm];

MakeSequenceBox[{f_,r__}] := 
  RowBox[Prepend[Map[RowBox[{", ",MakeBoxes[#,StandardForm]}]&,{r}],
      MakeBoxes[f,StandardForm]]];
      
MakeSequenceBox[{},Bracket -> Curly] = RowBox[{"{","}"}];

MakeSequenceBox[{expr_},Bracket -> Curly] := 
  RowBox[{"{",MakeBoxes[expr,StandardForm],"}"}];
  
MakeSequenceBox[{f_,r__},Bracket -> Curly] := 
  RowBox[{"{",
      RowBox[Prepend[Map[RowBox[{", ",MakeBoxes[#,StandardForm]}]&,{r}],
          MakeBoxes[f,StandardForm]]],"}"}];
          
MakeSequenceBox[{},Bracket -> Round] = RowBox[{"(",")"}];

MakeSequenceBox[{expr_},Bracket -> Round] := 
  RowBox[{"(",MakeBoxes[expr,StandardForm],")"}];
  
MakeSequenceBox[{f_,r__},Bracket -> Round] := 
  RowBox[{"(",
      RowBox[Prepend[Map[RowBox[{", ",MakeBoxes[#,StandardForm]}]&,{r}],
          MakeBoxes[f,StandardForm]]],")"}];
          
MakeSequenceBox[{},Bracket -> Square] = RowBox[{"[","]"}];

MakeSequenceBox[{expr_},Bracket -> Square] := 
  RowBox[{"[",MakeBoxes[expr,StandardForm],"]"}];
  
MakeSequenceBox[{f_,r__},Bracket -> Square] := 
  RowBox[{"[",
      RowBox[Prepend[Map[RowBox[{", ",MakeBoxes[#,StandardForm]}]&,{r}],
          MakeBoxes[f,StandardForm]]],"]"}];


(* Point Group Identification *)


PointGroup[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ
] := (PointGroupCycleStructure[s] /. PointGroupIdentificationRules);

PointGroup[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ, 
	pges:{__PointGroupElement}
] := (PointGroupCycleStructure[s,pges] /. PointGroupIdentificationRules);

PointGroup[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ,
	opts_Rule
] := (PointGroupCycleStructure[s,opts] /. PointGroupIdentificationRules);

PointGroup[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ, 
	pges:{__PointGroupElement},
	opts_Rule
] := (PointGroupCycleStructure[s,pges,opts] /. \
\
PointGroupIdentificationRules);



PointGroupReport[
	s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ,
	pges:{__PointGroupElement}
]:=
	Block[
		{pgcs},
		pgcs = PointGroupCycleStructure[s,pges];
		Print["The group cycles of the point group are:\n",pgcs[[1]]];
		Print["The corresponding point groups are: ",
   		DisplayForm[
      		MakeSequenceBox[pgcs[[1]]/.PointGroupIdentificationRules]
      	],
  		".\n"];
		If[
			pgcs[[1]]=!=pgcs[[2]],
			Print["The group cycles including conformational transformations are:\n", 
			pgcs[[2]]];
			Print["Such a group cycle structure would correspond to: ",
        DisplayForm[
        	MakeSequenceBox[pgcs[[2]]/.PointGroupIdentificationRules]
        ],
        ".\n"]
		]
	];
	
PointGroupReport[s:(sh_Symbol | sh_Symbol[__])?ParentCompoundQ]:=
	Block[
		{pgcs},
		pgcs = PointGroupCycleStructure[s];
		Print["The group cycles of the point group are:\n",pgcs[[1]]];
		Print["The corresponding point groups are: ",
      	DisplayForm[
      		MakeSequenceBox[pgcs[[1]]/.PointGroupIdentificationRules]
      	],
 		".\n"];
		If[
			pgcs[[1]]=!=pgcs[[2]],
			Print["The group cycles including conformational \
transformations are:\n", pgcs[[2]]];
			Print["Such a group cycle structure would correspond to: ",
        DisplayForm[
        	MakeSequenceBox[pgcs[[2]]/.PointGroupIdentificationRules]
        ],
        ".\n"]
		]
	];


(* Point Group Identification Rules *)

PointGroup/: MakeBoxes[PointGroup[],StandardForm] = Indeterminate;
PointGroup /: MakeBoxes[PointGroup[s_Symbol],StandardForm] :=
  MakeBoxes[#,StandardForm]&[SymbolName[s]];
PointGroup /: MakeBoxes[PointGroup[s_Symbol,n_Symbol],StandardForm] :=
  SubscriptBox[MakeBoxes[#1,StandardForm], MakeBoxes[#2,StandardForm]]&[
SymbolName[s],SymbolName[n]];
PointGroup /: MakeBoxes[PointGroup[s_Symbol,n_],StandardForm] :=
  SubscriptBox[MakeBoxes[#1,StandardForm], MakeBoxes[n,StandardForm]]&[
SymbolName[s]];
PointGroup /: MakeBoxes[PointGroup[s_Symbol,n_,m_Symbol],StandardForm] :=
  SubscriptBox[MakeBoxes[#1,StandardForm], 
    MakeBoxes[SequenceForm[n,#2],StandardForm]]&[SymbolName[s],SymbolName[m]];



PointGroupIdentificationRules = 
	Dispatch[{PointGroupCycleStructure[{PointGroupCycle[{1,1},False]}]->{
      PointGroup[C,1]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},False]}]->{PointGroup[C,2]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},True]}]->{PointGroup[C,i],PointGroup[C,s],
      PointGroup[S,2],PointGroup[C,1,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},False]},{PointGroupCycle[{3,1},False],
        PointGroupCycle[{3,2},False]}]->{PointGroup[D,3]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},True]},{PointGroupCycle[{3,1},False],
        PointGroupCycle[{3,2},False]}]->{PointGroup[C,3,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{3,1},False]},{PointGroupCycle[{3,2},False]}]->{
      PointGroup[C,3]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},False]},{PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{2,1},False]}]->{PointGroup[D,2]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},False]},{PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{2,1},True]}]->{PointGroup[C,2,h],
      PointGroup[C,2,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},False]},{2 PointGroupCycle[{3,1},False],
        2 PointGroupCycle[{3,2},False]},{2 PointGroupCycle[{3,1},False],
        2 PointGroupCycle[{3,2},False]}]->{PointGroup[T]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        5 PointGroupCycle[{2,1},False]},{PointGroupCycle[{5,1},False],
        PointGroupCycle[{5,4},False]},{PointGroupCycle[{5,2},False],
        PointGroupCycle[{5,3},False]}]->{PointGroup[D,5]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        5 PointGroupCycle[{2,1},True]},{PointGroupCycle[{5,1},False],
        PointGroupCycle[{5,4},False]},{PointGroupCycle[{5,2},False],
        PointGroupCycle[{5,3},False]}]->{PointGroup[C,5,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{4,1},False]},{PointGroupCycle[{4,2},False]},{
        PointGroupCycle[{4,3},False]}]->{PointGroup[C,4]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{4,1},True]},{PointGroupCycle[{4,2},False]},{
        PointGroupCycle[{4,3},True]}]->{PointGroup[S,4]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        2 PointGroupCycle[{2,1},False]},{2 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{4,2},False]},{PointGroupCycle[{4,1},False],
        PointGroupCycle[{4,3},False]}]->{PointGroup[D,4]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        2 PointGroupCycle[{2,1},False]},{2 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{4,2},False]},{PointGroupCycle[{4,1},True],
        PointGroupCycle[{4,3},True]}]->{PointGroup[D,2,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},False]},{3 PointGroupCycle[{4,2},False]},{
        4 PointGroupCycle[{3,1},False],4 PointGroupCycle[{3,2},False]},{
        3 PointGroupCycle[{4,1},False],3 PointGroupCycle[{4,3},False]}]
      ->{PointGroup[O]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        7 PointGroupCycle[{2,1},False]},{PointGroupCycle[{7,1},False],
        PointGroupCycle[{7,6},False]},{PointGroupCycle[{7,2},False],
        PointGroupCycle[{7,5},False]},{PointGroupCycle[{7,3},False],
        PointGroupCycle[{7,4},False]}]->{PointGroup[D,7]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        15 PointGroupCycle[{2,1},False]},{10 PointGroupCycle[{3,1},False],
        10 PointGroupCycle[{3,2},False]},{6 PointGroupCycle[{5,1},False],
        6 PointGroupCycle[{5,4},False]},{6 PointGroupCycle[{5,2},False],
        6 PointGroupCycle[{5,3},False]}]->{PointGroup[Y]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        2 PointGroupCycle[{2,1},True]},{2 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{4,2},False]},{PointGroupCycle[{4,1},False],
        PointGroupCycle[{4,3},False]}]->{PointGroup[C,4,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},True]},{3 PointGroupCycle[{4,2},False]},{
        4 PointGroupCycle[{3,1},False],4 PointGroupCycle[{3,2},False]},{
        3 PointGroupCycle[{4,1},True],3 PointGroupCycle[{4,3},True]}]->{
      PointGroup[T,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        7 PointGroupCycle[{2,1},True]},{PointGroupCycle[{7,1},False],
        PointGroupCycle[{7,6},False]},{PointGroupCycle[{7,2},False],
        PointGroupCycle[{7,5},False]},{PointGroupCycle[{7,3},False],
        PointGroupCycle[{7,4},False]}]->{PointGroup[C,7,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{5,1},False]},{PointGroupCycle[{5,2},False]},{
        PointGroupCycle[{5,3},False]},{PointGroupCycle[{5,4},False]}]->{
      PointGroup[C,5]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},False]},{3 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{6,3},False]},{PointGroupCycle[{6,1},False],
        PointGroupCycle[{6,5},False]},{PointGroupCycle[{6,2},False],
        PointGroupCycle[{6,4},False]}]->{PointGroup[D,6]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},False]},{3 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{6,3},True]},{PointGroupCycle[{6,1},True],
        PointGroupCycle[{6,5},True]},{PointGroupCycle[{6,2},False],
        PointGroupCycle[{6,4},False]}]->{PointGroup[D,3,d],
      PointGroup[D,3,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        9 PointGroupCycle[{2,1},False]},{PointGroupCycle[{9,1},False],
        PointGroupCycle[{9,8},False]},{PointGroupCycle[{9,2},False],
        PointGroupCycle[{9,7},False]},{PointGroupCycle[{9,3},False],
        PointGroupCycle[{9,6},False]},{PointGroupCycle[{9,4},False],
        PointGroupCycle[{9,5},False]}]->{PointGroup[D,9]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},True]},{3 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{6,3},False]},{PointGroupCycle[{6,1},False],
        PointGroupCycle[{6,5},False]},{PointGroupCycle[{6,2},False],
        PointGroupCycle[{6,4},False]}]->{PointGroup[C,6,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        9 PointGroupCycle[{2,1},True]},{PointGroupCycle[{9,1},False],
        PointGroupCycle[{9,8},False]},{PointGroupCycle[{9,2},False],
        PointGroupCycle[{9,7},False]},{PointGroupCycle[{9,3},False],
        PointGroupCycle[{9,6},False]},{PointGroupCycle[{9,4},False],
        PointGroupCycle[{9,5},False]}]->{PointGroup[C,9,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{6,1},False]},{PointGroupCycle[{6,2},False]},{
        PointGroupCycle[{6,3},False]},{PointGroupCycle[{6,4},False]},{
        PointGroupCycle[{6,5},False]}]->{PointGroup[C,6]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{6,1},True]},{PointGroupCycle[{6,2},False]},{
        PointGroupCycle[{6,3},True]},{PointGroupCycle[{6,4},False]},{
        PointGroupCycle[{6,5},True]}]->{PointGroup[S,6],
      PointGroup[C,3,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        4 PointGroupCycle[{2,1},False]},{4 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{8,4},False]},{PointGroupCycle[{8,1},False],
        PointGroupCycle[{8,7},False]},{PointGroupCycle[{8,2},False],
        PointGroupCycle[{8,6},False]},{PointGroupCycle[{8,3},False],
        PointGroupCycle[{8,5},False]}]->{PointGroup[D,8]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        4 PointGroupCycle[{2,1},False]},{4 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{8,4},False]},{PointGroupCycle[{8,1},True],
        PointGroupCycle[{8,7},True]},{PointGroupCycle[{8,2},False],
        PointGroupCycle[{8,6},False]},{PointGroupCycle[{8,3},True],
        PointGroupCycle[{8,5},True]}]->{PointGroup[D,4,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        11 PointGroupCycle[{2,1},False]},{PointGroupCycle[{11,1},False],
        PointGroupCycle[{11,10},False]},{PointGroupCycle[{11,2},False],
        PointGroupCycle[{11,9},False]},{PointGroupCycle[{11,3},False],
        PointGroupCycle[{11,8},False]},{PointGroupCycle[{11,4},False],
        PointGroupCycle[{11,7},False]},{PointGroupCycle[{11,5},False],
        PointGroupCycle[{11,6},False]}]->{PointGroup[D,11]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        4 PointGroupCycle[{2,1},True]},{4 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{8,4},False]},{PointGroupCycle[{8,1},False],
        PointGroupCycle[{8,7},False]},{PointGroupCycle[{8,2},False],
        PointGroupCycle[{8,6},False]},{PointGroupCycle[{8,3},False],
        PointGroupCycle[{8,5},False]}]->{PointGroup[C,8,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        11 PointGroupCycle[{2,1},True]},{PointGroupCycle[{11,1},False],
        PointGroupCycle[{11,10},False]},{PointGroupCycle[{11,2},False],
        PointGroupCycle[{11,9},False]},{PointGroupCycle[{11,3},False],
        PointGroupCycle[{11,8},False]},{PointGroupCycle[{11,4},False],
        PointGroupCycle[{11,7},False]},{PointGroupCycle[{11,5},False],
        PointGroupCycle[{11,6},False]}]->{PointGroup[C,11,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{7,1},False]},{PointGroupCycle[{7,2},False]},{
        PointGroupCycle[{7,3},False]},{PointGroupCycle[{7,4},False]},{
        PointGroupCycle[{7,5},False]},{PointGroupCycle[{7,6},False]}]->{
      PointGroup[C,7]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},False]},{PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{2,1},False]},{PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{2,1},True]},{PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{2,1},True]}]->{PointGroup[D,2,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},False]},{3 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{6,3},True]},{2 PointGroupCycle[{6,1},True],
        2 PointGroupCycle[{6,5},True]},{2 PointGroupCycle[{6,1},True],
        2 PointGroupCycle[{6,5},True]},{2 PointGroupCycle[{6,2},False],
        2 PointGroupCycle[{6,4},False]},{2 PointGroupCycle[{6,2},False],
        2 PointGroupCycle[{6,4},False]}]->{PointGroup[T,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        5 PointGroupCycle[{2,1},False]},{5 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{10,5},False]},{PointGroupCycle[{10,1},False],
        PointGroupCycle[{10,9},False]},{PointGroupCycle[{10,2},False],
        PointGroupCycle[{10,8},False]},{PointGroupCycle[{10,3},False],
        PointGroupCycle[{10,7},False]},{PointGroupCycle[{10,4},False],
        PointGroupCycle[{10,6},False]}]->{PointGroup[D,10]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        5 PointGroupCycle[{2,1},False]},{5 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{10,5},True]},{PointGroupCycle[{10,1},True],
        PointGroupCycle[{10,9},True]},{PointGroupCycle[{10,2},False],
        PointGroupCycle[{10,8},False]},{PointGroupCycle[{10,3},True],
        PointGroupCycle[{10,7},True]},{PointGroupCycle[{10,4},False],
        PointGroupCycle[{10,6},False]}]->{PointGroup[D,5,d],
      PointGroup[D,5,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},True]},{PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{4,1},False]},{PointGroupCycle[{4,1},True]},{
        PointGroupCycle[{4,2},False]},{PointGroupCycle[{4,3},False]},{
        PointGroupCycle[{4,3},True]}]->{PointGroup[C,4,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        5 PointGroupCycle[{2,1},True]},{5 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{10,5},False]},{PointGroupCycle[{10,1},False],
        PointGroupCycle[{10,9},False]},{PointGroupCycle[{10,2},False],
        PointGroupCycle[{10,8},False]},{PointGroupCycle[{10,3},False],
        PointGroupCycle[{10,7},False]},{PointGroupCycle[{10,4},False],
        PointGroupCycle[{10,6},False]}]->{PointGroup[C,10,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{8,1},False]},{PointGroupCycle[{8,2},False]},{
        PointGroupCycle[{8,3},False]},{PointGroupCycle[{8,4},False]},{
        PointGroupCycle[{8,5},False]},{PointGroupCycle[{8,6},False]},{
        PointGroupCycle[{8,7},False]}]->{PointGroup[C,8]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{8,1},True]},{PointGroupCycle[{8,2},False]},{
        PointGroupCycle[{8,3},True]},{PointGroupCycle[{8,4},False]},{
        PointGroupCycle[{8,5},True]},{PointGroupCycle[{8,6},False]},{
        PointGroupCycle[{8,7},True]}]->{PointGroup[S,8]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},False]},{6 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{12,6},False]},{PointGroupCycle[{12,1},False],
        PointGroupCycle[{12,11},False]},{PointGroupCycle[{12,2},False],
        PointGroupCycle[{12,10},False]},{PointGroupCycle[{12,3},False],
        PointGroupCycle[{12,9},False]},{PointGroupCycle[{12,4},False],
        PointGroupCycle[{12,8},False]},{PointGroupCycle[{12,5},False],
        PointGroupCycle[{12,7},False]}]->{PointGroup[D,12]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},False]},{6 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{12,6},False]},{PointGroupCycle[{12,1},True],
        PointGroupCycle[{12,11},True]},{PointGroupCycle[{12,2},False],
        PointGroupCycle[{12,10},False]},{PointGroupCycle[{12,3},True],
        PointGroupCycle[{12,9},True]},{PointGroupCycle[{12,4},False],
        PointGroupCycle[{12,8},False]},{PointGroupCycle[{12,5},True],
        PointGroupCycle[{12,7},True]}]->{PointGroup[D,6,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},True]},{6 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{12,6},False]},{PointGroupCycle[{12,1},False],
        PointGroupCycle[{12,11},False]},{PointGroupCycle[{12,2},False],
        PointGroupCycle[{12,10},False]},{PointGroupCycle[{12,3},False],
        PointGroupCycle[{12,9},False]},{PointGroupCycle[{12,4},False],
        PointGroupCycle[{12,8},False]},{PointGroupCycle[{12,5},False],
        PointGroupCycle[{12,7},False]}]->{PointGroup[C,12,v]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{9,1},False]},{PointGroupCycle[{9,2},False]},{
        PointGroupCycle[{9,3},False]},{PointGroupCycle[{9,4},False]},{
        PointGroupCycle[{9,5},False]},{PointGroupCycle[{9,6},False]},{
        PointGroupCycle[{9,7},False]},{PointGroupCycle[{9,8},False]}]->{
      PointGroup[C,9]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        2 PointGroupCycle[{2,1},False]},{2 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{2,1},True]},{PointGroupCycle[{2,1},True]},{
        2 PointGroupCycle[{2,1},True]},{2 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{4,2},False]},{PointGroupCycle[{4,1},False],
        PointGroupCycle[{4,3},False]},{PointGroupCycle[{4,1},True],
        PointGroupCycle[{4,3},True]}]->{PointGroup[D,4,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},False]},{3 PointGroupCycle[{2,1},True]},{
        6 PointGroupCycle[{2,1},True]},{3 PointGroupCycle[{4,2},False]},{
        PointGroupCycle[{6,3},True]},{3 PointGroupCycle[{4,1},False],
        3 PointGroupCycle[{4,3},False]},{3 PointGroupCycle[{4,1},True],
        3 PointGroupCycle[{4,3},True]},{4 PointGroupCycle[{6,1},True],
        4 PointGroupCycle[{6,5},True]},{4 PointGroupCycle[{6,2},False],
        4 PointGroupCycle[{6,4},False]}]->{PointGroup[O,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        7 PointGroupCycle[{2,1},False]},{7 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{14,7},True]},{PointGroupCycle[{14,1},True],
        PointGroupCycle[{14,13},True]},{PointGroupCycle[{14,2},False],
        PointGroupCycle[{14,12},False]},{PointGroupCycle[{14,3},True],
        PointGroupCycle[{14,11},True]},{PointGroupCycle[{14,4},False],
        PointGroupCycle[{14,10},False]},{PointGroupCycle[{14,5},True],
        PointGroupCycle[{14,9},True]},{PointGroupCycle[{14,6},False],
        PointGroupCycle[{14,8},False]}]->{PointGroup[D,7,d],
      PointGroup[D,7,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        15 PointGroupCycle[{2,1},False]},{15 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{10,5},True]},{10 PointGroupCycle[{6,1},True],
        10 PointGroupCycle[{6,5},True]},{10 PointGroupCycle[{6,2},False],
        10 PointGroupCycle[{6,4},False]},{6 PointGroupCycle[{10,1},True],
        6 PointGroupCycle[{10,9},True]},{6 PointGroupCycle[{10,2},False],
        6 PointGroupCycle[{10,8},False]},{6 PointGroupCycle[{10,3},True],
        6 PointGroupCycle[{10,7},True]},{6 PointGroupCycle[{10,4},False],
        6 PointGroupCycle[{10,6},False]}]->{PointGroup[Y,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{10,1},False]},{PointGroupCycle[{10,2},False]},{
        PointGroupCycle[{10,3},False]},{PointGroupCycle[{10,4},False]},{
        PointGroupCycle[{10,5},False]},{PointGroupCycle[{10,6},False]},{
        PointGroupCycle[{10,7},False]},{PointGroupCycle[{10,8},False]},{
        PointGroupCycle[{10,9},False]}]->{PointGroup[C,10]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{10,1},True]},{PointGroupCycle[{10,2},False]},{
        PointGroupCycle[{10,3},True]},{PointGroupCycle[{10,4},False]},{
        PointGroupCycle[{10,5},True]},{PointGroupCycle[{10,6},False]},{
        PointGroupCycle[{10,7},True]},{PointGroupCycle[{10,8},False]},{
        PointGroupCycle[{10,9},True]}]->{PointGroup[S,10],
      PointGroup[C,5,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        8 PointGroupCycle[{2,1},False]},{8 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{16,8},False]},{PointGroupCycle[{16,1},True],
        PointGroupCycle[{16,15},True]},{PointGroupCycle[{16,2},False],
        PointGroupCycle[{16,14},False]},{PointGroupCycle[{16,3},True],
        PointGroupCycle[{16,13},True]},{PointGroupCycle[{16,4},False],
        PointGroupCycle[{16,12},False]},{PointGroupCycle[{16,5},True],
        PointGroupCycle[{16,11},True]},{PointGroupCycle[{16,6},False],
        PointGroupCycle[{16,10},False]},{PointGroupCycle[{16,7},True],
        PointGroupCycle[{16,9},True]}]->{PointGroup[D,8,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{11,1},False]},{PointGroupCycle[{11,2},False]},{
        PointGroupCycle[{11,3},False]},{PointGroupCycle[{11,4},False]},{
        PointGroupCycle[{11,5},False]},{PointGroupCycle[{11,6},False]},{
        PointGroupCycle[{11,7},False]},{PointGroupCycle[{11,8},False]},{
        PointGroupCycle[{11,9},False]},{PointGroupCycle[{11,10},False]}]
      ->{PointGroup[C,11]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        3 PointGroupCycle[{2,1},False]},{3 PointGroupCycle[{2,1},False]},{
        3 PointGroupCycle[{2,1},True]},{3 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{6,3},False]},{PointGroupCycle[{6,3},True]},{
        PointGroupCycle[{6,3},True]},{PointGroupCycle[{6,1},False],
        PointGroupCycle[{6,5},False]},{PointGroupCycle[{6,1},True],
        PointGroupCycle[{6,5},True]},{PointGroupCycle[{6,1},True],
        PointGroupCycle[{6,5},True]},{PointGroupCycle[{6,2},False],
        PointGroupCycle[{6,4},False]}]->{PointGroup[D,6,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        9 PointGroupCycle[{2,1},False]},{9 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{18,9},True]},{PointGroupCycle[{18,1},True],
        PointGroupCycle[{18,17},True]},{PointGroupCycle[{18,2},False],
        PointGroupCycle[{18,16},False]},{PointGroupCycle[{18,3},True],
        PointGroupCycle[{18,15},True]},{PointGroupCycle[{18,4},False],
        PointGroupCycle[{18,14},False]},{PointGroupCycle[{18,5},True],
        PointGroupCycle[{18,13},True]},{PointGroupCycle[{18,6},False],
        PointGroupCycle[{18,12},False]},{PointGroupCycle[{18,7},True],
        PointGroupCycle[{18,11},True]},{PointGroupCycle[{18,8},False],
        PointGroupCycle[{18,10},False]}]->{PointGroup[D,9,d],
      PointGroup[D,9,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{6,1},False]},{PointGroupCycle[{6,1},True]},{
        PointGroupCycle[{6,1},True]},{PointGroupCycle[{6,2},False]},{
        PointGroupCycle[{6,3},False]},{PointGroupCycle[{6,3},True]},{
        PointGroupCycle[{6,3},True]},{PointGroupCycle[{6,4},False]},{
        PointGroupCycle[{6,5},False]},{PointGroupCycle[{6,5},True]},{
        PointGroupCycle[{6,5},True]}]->{PointGroup[C,6,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{12,1},False]},{PointGroupCycle[{12,2},False]},{
        PointGroupCycle[{12,3},False]},{PointGroupCycle[{12,4},False]},{
        PointGroupCycle[{12,5},False]},{PointGroupCycle[{12,6},False]},{
        PointGroupCycle[{12,7},False]},{PointGroupCycle[{12,8},False]},{
        PointGroupCycle[{12,9},False]},{PointGroupCycle[{12,10},False]},{
        PointGroupCycle[{12,11},False]}]->{PointGroup[C,12]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{12,1},True]},{PointGroupCycle[{12,2},False]},{
        PointGroupCycle[{12,3},True]},{PointGroupCycle[{12,4},False]},{
        PointGroupCycle[{12,5},True]},{PointGroupCycle[{12,6},False]},{
        PointGroupCycle[{12,7},True]},{PointGroupCycle[{12,8},False]},{
        PointGroupCycle[{12,9},True]},{PointGroupCycle[{12,10},False]},{
        PointGroupCycle[{12,11},True]}]->{PointGroup[S,12]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        10 PointGroupCycle[{2,1},False]},{10 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{20,10},False]},{PointGroupCycle[{20,1},True],
        PointGroupCycle[{20,19},True]},{PointGroupCycle[{20,2},False],
        PointGroupCycle[{20,18},False]},{PointGroupCycle[{20,3},True],
        PointGroupCycle[{20,17},True]},{PointGroupCycle[{20,4},False],
        PointGroupCycle[{20,16},False]},{PointGroupCycle[{20,5},True],
        PointGroupCycle[{20,15},True]},{PointGroupCycle[{20,6},False],
        PointGroupCycle[{20,14},False]},{PointGroupCycle[{20,7},True],
        PointGroupCycle[{20,13},True]},{PointGroupCycle[{20,8},False],
        PointGroupCycle[{20,12},False]},{PointGroupCycle[{20,9},True],
        PointGroupCycle[{20,11},True]}]->{PointGroup[D,10,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        4 PointGroupCycle[{2,1},False]},{4 PointGroupCycle[{2,1},False]},{
        PointGroupCycle[{2,1},True]},{PointGroupCycle[{2,1},True]},{
        4 PointGroupCycle[{2,1},True]},{4 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{8,4},False]},{PointGroupCycle[{4,1},True],
        PointGroupCycle[{4,3},True]},{PointGroupCycle[{8,1},False],
        PointGroupCycle[{8,7},False]},{PointGroupCycle[{8,1},True],
        PointGroupCycle[{8,7},True]},{PointGroupCycle[{8,2},False],
        PointGroupCycle[{8,6},False]},{PointGroupCycle[{8,3},False],
        PointGroupCycle[{8,5},False]},{PointGroupCycle[{8,3},True],
        PointGroupCycle[{8,5},True]}]->{PointGroup[D,8,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        11 PointGroupCycle[{2,1},False]},{11 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{22,11},True]},{PointGroupCycle[{22,1},True],
        PointGroupCycle[{22,21},True]},{PointGroupCycle[{22,2},False],
        PointGroupCycle[{22,20},False]},{PointGroupCycle[{22,3},True],
        PointGroupCycle[{22,19},True]},{PointGroupCycle[{22,4},False],
        PointGroupCycle[{22,18},False]},{PointGroupCycle[{22,5},True],
        PointGroupCycle[{22,17},True]},{PointGroupCycle[{22,6},False],
        PointGroupCycle[{22,16},False]},{PointGroupCycle[{22,7},True],
        PointGroupCycle[{22,15},True]},{PointGroupCycle[{22,8},False],
        PointGroupCycle[{22,14},False]},{PointGroupCycle[{22,9},True],
        PointGroupCycle[{22,13},True]},{PointGroupCycle[{22,10},False],
        PointGroupCycle[{22,12},False]}]->{PointGroup[D,11,d],
      PointGroup[D,11,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{14,1},True]},{PointGroupCycle[{14,2},False]},{
        PointGroupCycle[{14,3},True]},{PointGroupCycle[{14,4},False]},{
        PointGroupCycle[{14,5},True]},{PointGroupCycle[{14,6},False]},{
        PointGroupCycle[{14,7},True]},{PointGroupCycle[{14,8},False]},{
        PointGroupCycle[{14,9},True]},{PointGroupCycle[{14,10},False]},{
        PointGroupCycle[{14,11},True]},{PointGroupCycle[{14,12},False]},{
        PointGroupCycle[{14,13},True]}]->{PointGroup[S,14],
      PointGroup[C,7,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        12 PointGroupCycle[{2,1},False]},{12 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{24,12},False]},{PointGroupCycle[{24,1},True],
        PointGroupCycle[{24,23},True]},{PointGroupCycle[{24,2},False],
        PointGroupCycle[{24,22},False]},{PointGroupCycle[{24,3},True],
        PointGroupCycle[{24,21},True]},{PointGroupCycle[{24,4},False],
        PointGroupCycle[{24,20},False]},{PointGroupCycle[{24,5},True],
        PointGroupCycle[{24,19},True]},{PointGroupCycle[{24,6},False],
        PointGroupCycle[{24,18},False]},{PointGroupCycle[{24,7},True],
        PointGroupCycle[{24,17},True]},{PointGroupCycle[{24,8},False],
        PointGroupCycle[{24,16},False]},{PointGroupCycle[{24,9},True],
        PointGroupCycle[{24,15},True]},{PointGroupCycle[{24,10},False],
        PointGroupCycle[{24,14},False]},{PointGroupCycle[{24,11},True],
        PointGroupCycle[{24,13},True]}]->{PointGroup[D,12,d]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        5 PointGroupCycle[{2,1},False]},{5 PointGroupCycle[{2,1},False]},{
        5 PointGroupCycle[{2,1},True]},{5 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{10,5},False]},{PointGroupCycle[{10,5},True]},{
        PointGroupCycle[{10,5},True]},{PointGroupCycle[{10,1},False],
        PointGroupCycle[{10,9},False]},{PointGroupCycle[{10,1},True],
        PointGroupCycle[{10,9},True]},{PointGroupCycle[{10,1},True],
        PointGroupCycle[{10,9},True]},{PointGroupCycle[{10,2},False],
        PointGroupCycle[{10,8},False]},{PointGroupCycle[{10,3},False],
        PointGroupCycle[{10,7},False]},{PointGroupCycle[{10,3},True],
        PointGroupCycle[{10,7},True]},{PointGroupCycle[{10,3},True],
        PointGroupCycle[{10,7},True]},{PointGroupCycle[{10,4},False],
        PointGroupCycle[{10,6},False]}]->{PointGroup[D,10,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{2,1},True]},{PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{4,1},True]},{PointGroupCycle[{4,3},True]},{
        PointGroupCycle[{8,1},False]},{PointGroupCycle[{8,1},True]},{
        PointGroupCycle[{8,2},False]},{PointGroupCycle[{8,3},False]},{
        PointGroupCycle[{8,3},True]},{PointGroupCycle[{8,4},False]},{
        PointGroupCycle[{8,5},False]},{PointGroupCycle[{8,5},True]},{
        PointGroupCycle[{8,6},False]},{PointGroupCycle[{8,7},False]},{
        PointGroupCycle[{8,7},True]}]->{PointGroup[C,8,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{16,1},True]},{PointGroupCycle[{16,2},False]},{
        PointGroupCycle[{16,3},True]},{PointGroupCycle[{16,4},False]},{
        PointGroupCycle[{16,5},True]},{PointGroupCycle[{16,6},False]},{
        PointGroupCycle[{16,7},True]},{PointGroupCycle[{16,8},False]},{
        PointGroupCycle[{16,9},True]},{PointGroupCycle[{16,10},False]},{
        PointGroupCycle[{16,11},True]},{PointGroupCycle[{16,12},False]},{
        PointGroupCycle[{16,13},True]},{PointGroupCycle[{16,14},False]},{
        PointGroupCycle[{16,15},True]}]->{PointGroup[S,16]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        6 PointGroupCycle[{2,1},False]},{6 PointGroupCycle[{2,1},False]},{
        6 PointGroupCycle[{2,1},True]},{6 PointGroupCycle[{2,1},True]},{
        PointGroupCycle[{6,3},True]},{PointGroupCycle[{6,3},True]},{
        PointGroupCycle[{12,6},False]},{PointGroupCycle[{6,1},True],
        PointGroupCycle[{6,5},True]},{PointGroupCycle[{6,1},True],
        PointGroupCycle[{6,5},True]},{PointGroupCycle[{12,1},False],
        PointGroupCycle[{12,11},False]},{PointGroupCycle[{12,1},True],
        PointGroupCycle[{12,11},True]},{PointGroupCycle[{12,2},False],
        PointGroupCycle[{12,10},False]},{PointGroupCycle[{12,3},False],
        PointGroupCycle[{12,9},False]},{PointGroupCycle[{12,3},True],
        PointGroupCycle[{12,9},True]},{PointGroupCycle[{12,4},False],
        PointGroupCycle[{12,8},False]},{PointGroupCycle[{12,5},False],
        PointGroupCycle[{12,7},False]},{PointGroupCycle[{12,5},True],
        PointGroupCycle[{12,7},True]}]->{PointGroup[D,12,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{18,1},True]},{PointGroupCycle[{18,2},False]},{
        PointGroupCycle[{18,3},True]},{PointGroupCycle[{18,4},False]},{
        PointGroupCycle[{18,5},True]},{PointGroupCycle[{18,6},False]},{
        PointGroupCycle[{18,7},True]},{PointGroupCycle[{18,8},False]},{
        PointGroupCycle[{18,9},True]},{PointGroupCycle[{18,10},False]},{
        PointGroupCycle[{18,11},True]},{PointGroupCycle[{18,12},False]},{
        PointGroupCycle[{18,13},True]},{PointGroupCycle[{18,14},False]},{
        PointGroupCycle[{18,15},True]},{PointGroupCycle[{18,16},False]},{
        PointGroupCycle[{18,17},True]}]->{PointGroup[S,18],
      PointGroup[C,9,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{10,1},False]},{PointGroupCycle[{10,1},True]},{
        PointGroupCycle[{10,1},True]},{PointGroupCycle[{10,2},False]},{
        PointGroupCycle[{10,3},False]},{PointGroupCycle[{10,3},True]},{
        PointGroupCycle[{10,3},True]},{PointGroupCycle[{10,4},False]},{
        PointGroupCycle[{10,5},False]},{PointGroupCycle[{10,5},True]},{
        PointGroupCycle[{10,5},True]},{PointGroupCycle[{10,6},False]},{
        PointGroupCycle[{10,7},False]},{PointGroupCycle[{10,7},True]},{
        PointGroupCycle[{10,7},True]},{PointGroupCycle[{10,8},False]},{
        PointGroupCycle[{10,9},False]},{PointGroupCycle[{10,9},True]},{
        PointGroupCycle[{10,9},True]}]->{PointGroup[C,10,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{20,1},True]},{PointGroupCycle[{20,2},False]},{
        PointGroupCycle[{20,3},True]},{PointGroupCycle[{20,4},False]},{
        PointGroupCycle[{20,5},True]},{PointGroupCycle[{20,6},False]},{
        PointGroupCycle[{20,7},True]},{PointGroupCycle[{20,8},False]},{
        PointGroupCycle[{20,9},True]},{PointGroupCycle[{20,10},False]},{
        PointGroupCycle[{20,11},True]},{PointGroupCycle[{20,12},False]},{
        PointGroupCycle[{20,13},True]},{PointGroupCycle[{20,14},False]},{
        PointGroupCycle[{20,15},True]},{PointGroupCycle[{20,16},False]},{
        PointGroupCycle[{20,17},True]},{PointGroupCycle[{20,18},False]},{
        PointGroupCycle[{20,19},True]}]->{PointGroup[S,20]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{22,1},True]},{PointGroupCycle[{22,2},False]},{
        PointGroupCycle[{22,3},True]},{PointGroupCycle[{22,4},False]},{
        PointGroupCycle[{22,5},True]},{PointGroupCycle[{22,6},False]},{
        PointGroupCycle[{22,7},True]},{PointGroupCycle[{22,8},False]},{
        PointGroupCycle[{22,9},True]},{PointGroupCycle[{22,10},False]},{
        PointGroupCycle[{22,11},True]},{PointGroupCycle[{22,12},False]},{
        PointGroupCycle[{22,13},True]},{PointGroupCycle[{22,14},False]},{
        PointGroupCycle[{22,15},True]},{PointGroupCycle[{22,16},False]},{
        PointGroupCycle[{22,17},True]},{PointGroupCycle[{22,18},False]},{
        PointGroupCycle[{22,19},True]},{PointGroupCycle[{22,20},False]},{
        PointGroupCycle[{22,21},True]}]->{PointGroup[S,22],
      PointGroup[C,11,h]},
  PointGroupCycleStructure[{PointGroupCycle[{1,1},False]},{
        PointGroupCycle[{6,1},True]},{PointGroupCycle[{6,1},True]},{
        PointGroupCycle[{6,3},True]},{PointGroupCycle[{6,3},True]},{
        PointGroupCycle[{6,5},True]},{PointGroupCycle[{6,5},True]},{
        PointGroupCycle[{12,1},False]},{PointGroupCycle[{12,1},True]},{
        PointGroupCycle[{12,2},False]},{PointGroupCycle[{12,3},False]},{
        PointGroupCycle[{12,3},True]},{PointGroupCycle[{12,4},False]},{
        PointGroupCycle[{12,5},False]},{PointGroupCycle[{12,5},True]},{
        PointGroupCycle[{12,6},False]},{PointGroupCycle[{12,7},False]},{
        PointGroupCycle[{12,7},True]},{PointGroupCycle[{12,8},False]},{
        PointGroupCycle[{12,9},False]},{PointGroupCycle[{12,9},True]},{
        PointGroupCycle[{12,10},False]},{PointGroupCycle[{12,11},False]},{
        PointGroupCycle[{12,11},True]}]->{PointGroup[C,12,h]},
	PointGroupCycleStructure[__]->{PointGroup[]}}];



(* Polya Generating Functions *)

(* Cycle Index *)

CycleOf[perm_List,n_Integer] :=
	Drop[FixedPointList[
			Part[perm,#]&,
			n,
			SameTest -> (SameQ[n, #2]&)
	],-1]/; MemberQ[perm,n];
	
ToCycles[perm_List] := 
	Block[
		{cycls = {},elems = perm},
		While[
			elems != {},
			AppendTo[cycls, CycleOf[perm,First[elems]]];
			elems = Fold[DeleteCases,elems,Last[cycls]]
		];
		Return[cycls]
	];
	
PolyaVariable /: MakeBoxes[PolyaVariable[n_,p_],StandardForm]:=
	SubsuperscriptBox["\[ScriptF]",MakeBoxes[n,StandardForm],
    MakeBoxes[p,StandardForm]];
    
CycleIndices[pges:{__PointGroupElement}]:=
	Map[
		Apply[Plus,Apply[Times,Apply[PolyaVariable[#2,#1]&,
			Map[
				Map[
					{Length[#],First[#]}&,
					Split[Sort[Map[Length,ToCycles[First[#]]]]]
				]&,
				pges
			][[#]],{2}],{1}]]&,
		SubPointGroupIndices[pges],
		{2}
	];
	
Options[CycleIndex] = {Conformations-> True,Reflections-> False};

CycleIndex[pgesmol:{__PointGroupElement}|_?ParentCompoundQ,opts:___Rule] :=
	Together[#/(#/.PolyaVariable[__]->1)]&[
		({1,Reflections}.CycleIndices[pgesmol].{1,Conformations}/.
				{opts}/.Options[CycleIndex])/.{True->1,False->0}];


(* Polya Polynomial *)

Options[PolyaPolynomial] = {Conformations-> True,Reflections-> False};

PolyaPolynomial[pges:{__PointGroupElement},vars:{__},opts:___Rule] :=
	CycleIndex[pges,opts,Sequence@@Options[PolyaPolynomial]] /. 
	PolyaVariable[n_,p_]:> Apply[Plus,vars^n]^p;
       
PolyaPolynomial[mol_?ParentCompoundQ,vars:{__},opts:___Rule] :=
	CycleIndex[mol,opts,Sequence@@Options[PolyaPolynomial]] /. 
	PolyaVariable[n_,p_]:> Apply[Plus,vars^n]^p;


(* Polya Coefficients *)

PolyaCoefficient[m:{__Integer?NonNegative},0]  = 0;

PolyaCoefficient[m:{__Integer?NonNegative},p_PolyaVariable]  :=
	If[
		Apply[Plus,m] ==Apply[Times,p],
		PolyaCoefficientSR[m,{p}],
		0
	];
	
PolyaCoefficient[m:{__Integer?NonNegative},
    HoldPattern[Times[p__PolyaVariable]]]  :=
	If[
		Apply[Plus,m] ==Apply[Plus,Apply[Times,{p},{1}]],
		PolyaCoefficientSR[m,{p}],
		0
	];
	
PolyaCoefficientSR[m_,{PolyaVariable[k_,ak_]}] := 
	Apply[
		Plus,
		Apply[Multinomial,LimitedPartitions[ak,Floor[m/k]],{1}]
	];
	
PolyaCoefficientSR[m_,{ar__,PolyaVariable[k_,ak_]}] := 
	Apply[
		Plus,
		Apply[
			(Multinomial[##] PolyaCoefficientSR[m-k {##},{ar}])&,
			LimitedPartitions[ak,Floor[m/k]],
			{1}
		]
	];


(* Substituents Multiplicity *)

SubstituentsMultiplicity[s:{__Integer?Positive},
    n:(_Integer?Positive|_Symbol)]:=
	With[{ls = Map[Length,Split[s]]},
		Apply[
			Times,
			MapThread[
				Binomial,
				{FoldList[(#1- #2)&,n,Drop[ls,-1]],ls}
			]
		]
	];


(* Combinatoric Utility Functions *)

(* Partitions *)

Partitions[s_Integer?NonNegative] := 
	Level[PartitionsSR[s,s,{}],{-2}];

Partitions[s_Integer?NonNegative,m_Integer?NonNegative] := 
	Level[PartitionsSR[s,Min[s,m],{}],{-2}];
	
Partitions[s_Integer?NonNegative,{b_Integer?NonNegative,t_Integer?
NonNegative}]:=
	Level[Table[Map[Prepend[#,i]&,Partitions[n-i,i]],{i,Max[b,1],t}],{-2}];

Partitions[(s_Integer)?NonNegative, (m_Integer)?NonNegative, Initial -> e_] :=
	Level[PartitionsSR[s, Min[s, m], {e}], {-2}]

Partitions[s_Integer?NonNegative,m_Integer?NonNegative,n_Integer?Positive] :=
	Level[PartitionsSR2[s,Min[s,m],Min[s,n],{}],{-2}] /; s<=m*n;

PartitionsSR[0,_,pl_] := pl;

PartitionsSR[s_,1,pl_] := Join[pl,Table[1,{s}]];

PartitionsSR[s_,m_,pl_] := 
	Table[
		PartitionsSR[s-i,Min[i,s-i],Append[pl,i]],
		{i,m}
	];

PartitionsSR2[0,0,_,pl_] := pl;

PartitionsSR2[s_,m_,n_,pl_] := 
	Table[
		PartitionsSR2[s-i,Min[i,s-i],n-1,Append[pl,i]],
		{i,Ceiling[s/n],m}
	];


(* Limited Partitions *)

LimitedPartitions[n_Integer?Positive,l:{__Integer?NonNegative}] :=
	If[n <= Apply[Plus,l],
		Flatten[LimitedPartitionsSR[n,l,{}],Length[l]-1],
		{}
	];

LimitedPartitionsSR[n_,{l_},{as___}] := {{as,n}};

LimitedPartitionsSR[n_,{lf_,lr__},{as___}] :=
	Table[
		LimitedPartitionsSR[n-a,{lr},{as,a}],
		{a,Max[0,n-Plus[lr]],Min[n,lf]}
	];


(* Linear Map *)

LinearMap[f_,r_?NumberQ*p_] :=r*LinearMap[f,p];
LinearMap[f_,p_Plus] :=Map[LinearMap[f,#]&,p];
LinearMap[f_,p_] :=f[p];
LinearMap[f_,r_?NumberQ] :=r;


(* Polya Substitution Command *)

PolyaSubstitution[n_,s_,PolyaVariable[b_,p_]] :=
	If[Mod[s,b]==0 && s <= b p, 
  		Binomial[p,s/b]n^(s/b),
  		0
  	];
  
PolyaSubstitution[n_,s_,r__PolyaVariable PolyaVariable[b_,p_]] :=
	Sum[
		PolyaSubstitution[n,s-i b,Times[r]] * Binomial[p,i] * n^i,
		{i,0,Min[Floor[s/b],p]}
	];
	
PolyaSubstitutionRules[n_,ss_] := 
	{HoldPattern[Times[pvs__PolyaVariable]]:>
		PolyaSubstitution[n,ss,Times[pvs]],
	 pv_PolyaVariable :> PolyaSubstitution[n,ss,pv]};
      
      
(* Enumeration Commands *)

(* Number of Isomers *)

Options[NumberOfIsomers]= {Conformations-> True,Reflections-> False};

NumberOfIsomers[mol_?ParentCompoundQ,s:{__Integer?Positive},opts___Rule]:=
	If[ChiralParentCompoundQ[mol],2,1] * 
	LinearMap[
		PolyaCoefficient[
			Prepend[s,NumberOfSubstituentSites[mol]-Plus@@s],#]&,
		CycleIndex[mol,opts,Sequence@@Options[NumberOfIsomers]]
	] /; Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfIsomers[mol_?ParentCompoundQ,s:{__Integer?Positive},
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	If[ChiralParentCompoundQ[mol],2,1] * 
	If[Head[n]===Symbol,Simplify[Collect[#,n]],#]&[
		SubstituentsMultiplicity[s,n] * NumberOfIsomers[mol,s,opts]
	] /; Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfIsomers[mol_?ParentCompoundQ,ss_Integer?Positive,opts___Rule]:=
	Block[
		{sd = NumberOfSubstituentSites[mol]-ss},
		NumberOfIsomersSR[0,0,pl_] =
			LinearMap[
				PolyaCoefficient[pl,#]&,
				CycleIndex[mol,opts,Sequence@@Options[NumberOfIsomers]]
			];
		Return[
			If[ChiralParentCompoundQ[mol],2,1] * 
			NumberOfIsomersSR[ss,ss,If[Positive[sd],{sd},{}]]
		]
	] /; ss <= NumberOfSubstituentSites[mol];
	
NumberOfIsomers[mol_?ParentCompoundQ,ss_Integer?Positive,
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	Simplify[
		CycleIndex[mol,opts,Sequence@@Options[NumberOfIsomers]] /. 
			PolyaSubstitutionRules[n,ss]
	] /; ss <= NumberOfSubstituentSites[mol];
	
NumberOfIsomers[mol_?(Not[ParentCompoundQ[#]]&),__]:=
	Message[PointGroupGenerators::notdef,mol];
	
	
(* Number of Isomers Subroutine *)

NumberOfIsomersSR[s_,m_,pl_] := 
	Sum[
		NumberOfIsomersSR[s-i,Min[i,s-i],Append[pl,i]],
		{i,m}
	];


(* Number of Enantiomeric Pairs *)

Options[NumberOfEnantiomericPairs]= {Conformations-> True};

NumberOfEnantiomericPairs[mol_?ChiralParentCompoundQ,exprs___] := 
	NumberOfIsomers[mol,exprs]/2;

NumberOfEnantiomericPairs[mol_?ParentCompoundQ,s:{__Integer?Positive},
    opts___Rule]:=
	LinearMap[
		PolyaCoefficient[
			Prepend[s,NumberOfSubstituentSites[mol]-Plus@@s],#]&,
		Together[
			CycleIndex[mol,Reflections -> False,opts,
        		Sequence@@Options[NumberOfEnantiomericPairs]]-
			CycleIndex[mol,Reflections -> True,opts,
        		Sequence@@Options[NumberOfEnantiomericPairs]]
		]
	] /; Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfEnantiomericPairs[mol_?ParentCompoundQ,s:{__Integer?Positive},
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	If[
		Head[n]===Symbol,
		Simplify[Collect[#,n]],
		#
	]&[SubstituentsMultiplicity[s,n] * 
		NumberOfEnantiomericPairs[mol,s,opts]] /;
	Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfEnantiomericPairs[mol_?ParentCompoundQ,ss_Integer?Positive,
    opts___Rule]:=
	Block[
		{sd = NumberOfSubstituentSites[mol]-ss},
		NumberOfIsomersSR[0,0,pl_]  =
			LinearMap[
				PolyaCoefficient[pl,#]&,
				Together[
					CycleIndex[mol,Reflections -> False,opts,
                		Sequence@@Options[NumberOfEnantiomericPairs]]-
					CycleIndex[mol,Reflections -> True,opts,
                		Sequence@@Options[NumberOfEnantiomericPairs]]
				]
			];
		Return[NumberOfIsomersSR[ss,ss,If[Positive[sd],{sd},{}]]]
	] /; ss <= NumberOfSubstituentSites[mol];
	
NumberOfEnantiomericPairs[mol_?ParentCompoundQ,ss_Integer?Positive,
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	Simplify[Simplify[
		CycleIndex[mol,Reflections -> False,opts,
        	Sequence@@Options[NumberOfEnantiomericPairs]] -
		CycleIndex[mol,Reflections -> True,opts,
            Sequence@@Options[NumberOfEnantiomericPairs]]
       	] /. PolyaSubstitutionRules[n,ss]
	] /; ss <= NumberOfSubstituentSites[mol];

	
NumberOfEnantiomericPairs[mol_?(Not[ParentCompoundQ[#]]&),__]:=
	Message[PointGroupGenerators::notdef,mol];


(* Number of Achiral Isomers *)

Options[NumberOfAchiralIsomers]= {Conformations-> True};

NumberOfAchiralIsomers[mol_?ChiralParentCompoundQ,___] = 0;

NumberOfAchiralIsomers[mol_?ParentCompoundQ,s:{__Integer?Positive},
    opts___Rule]:=
	LinearMap[
		PolyaCoefficient[
			Prepend[s,NumberOfSubstituentSites[mol]-Plus@@s],#]&,
		Together[
			2 CycleIndex[mol,Reflections -> True,opts,
            	Sequence@@Options[NumberOfAchiralIsomers]]-
			CycleIndex[mol,Reflections -> False,opts,
            	Sequence@@Options[NumberOfAchiralIsomers]]
		]
	] /; Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfAchiralIsomers[mol_?ParentCompoundQ,s:{__Integer?Positive},
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	If[
		Head[n]===Symbol,
		Simplify[Collect[#,n]],
		#
	]&[SubstituentsMultiplicity[s,n] * 
		NumberOfAchiralIsomers[mol,s,opts]] /;
	Apply[Plus,s] <= NumberOfSubstituentSites[mol];

NumberOfAchiralIsomers[
	mol_?ParentCompoundQ,
	ss_Integer?Positive,
	opts___Rule
]:=
	Block[
		{sd = NumberOfSubstituentSites[mol]-ss},
		NumberOfIsomersSR[0,0,pl_] =
			LinearMap[
				PolyaCoefficient[pl,#]&,
				Together[
					2 CycleIndex[mol,Reflections -> True,opts,
       					Sequence@@Options[NumberOfAchiralIsomers]]-
					CycleIndex[mol,Reflections -> False,opts,
    					Sequence@@Options[NumberOfAchiralIsomers]]
				]
			];
		Return[NumberOfIsomersSR[ss,ss,If[Positive[sd],{sd},{}]]]
	] /; ss <= NumberOfSubstituentSites[mol];

NumberOfAchiralIsomers[
	mol_?ParentCompoundQ,
	ss_Integer?Positive,
    n:(_Integer?Positive|_Symbol),
    opts___Rule
]:=
	Simplify[Simplify[
		2 CycleIndex[mol,Reflections -> True,opts,
       		Sequence@@Options[NumberOfAchiralIsomers]] -
		CycleIndex[mol,Reflections -> False,opts,
    		Sequence@@Options[NumberOfAchiralIsomers]] 
    	] /. PolyaSubstitutionRules[n,ss]
	] /; ss <= NumberOfSubstituentSites[mol];
	
NumberOfAchiralIsomers[mol_?(Not[ParentCompoundQ[#]]&),__]:=
	Message[PointGroupGenerators::notdef,mol];


(* Number of Diastereoisomers *)

Options[NumberOfDiastereoisomers]= {Conformations-> True};

NumberOfDiastereoisomers[mol_?ParentCompoundQ,s:{__Integer?Positive},
    opts___Rule]:=
	LinearMap[
		PolyaCoefficient[
			Prepend[s,NumberOfSubstituentSites[mol]-Plus@@s],#]&,
		CycleIndex[mol,Reflections->True,opts,
        Sequence@@Options[NumberOfDiastereoisomers]]
	] /; Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfDiastereoisomers[mol_?ParentCompoundQ,s:{__Integer?Positive},
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	If[
		Head[n]===Symbol,
		Simplify[Collect[#,n]],
		#
	]&[SubstituentsMultiplicity[s,n] * 
		NumberOfDiastereoisomers[mol,s,opts]] /; 
	Apply[Plus,s] <= NumberOfSubstituentSites[mol];
	
NumberOfDiastereoisomers[mol_?ParentCompoundQ,ss_Integer?Positive,
    opts___Rule]:=
	Block[
		{sd = NumberOfSubstituentSites[mol]-ss},
		NumberOfIsomersSR[0,0,pl_] =
			LinearMap[
				PolyaCoefficient[pl,#]&,
				CycleIndex[mol,Reflections->True,opts,
        			Sequence@@Options[NumberOfDiastereoisomers]]
			];
		Return[NumberOfIsomersSR[ss,ss,If[Positive[sd],{sd},{}]]]
	] /; ss <= NumberOfSubstituentSites[mol];
	
NumberOfDiastereoisomers[mol_?ParentCompoundQ,ss_Integer?Positive,
    n:(_Integer?Positive|_Symbol),opts___Rule]:=
	Simplify[Simplify[
		2 CycleIndex[mol,Reflections -> True,opts,
       		Sequence@@Options[NumberOfAchiralIsomers]] -
		CycleIndex[mol,Reflections -> False,opts,
    		Sequence@@Options[NumberOfAchiralIsomers]]
    	] /. PolyaSubstitutionRules[n,ss]	    
	] /; ss <= NumberOfSubstituentSites[mol];
	
NumberOfDiastereoisomers[mol_?(Not[ParentCompoundQ[#]]&),__]:=
	Message[PointGroupGenerators::notdef,mol];
	
	
(* Panel Support *)

(* Database Panel *)

SortedParentCompounds := 
	Split[
		Sort[
			Sort[ParentCompounds],
			(NumberOfSubstituentSites[#1]<=NumberOfSubstituentSites[#2])&
		],
		(NumberOfSubstituentSites[#1]==NumberOfSubstituentSites[#2])&
	];
	
ParentCompoundButtonCell[mols:{__?ParentCompoundQ}] := 
	Cell[
		BoxData[StyleBox[
				GridBox[
					Map[{ButtonBox[ToBoxes[#],ButtonData:>#]}&,mols],
					RowSpacings->0, ColumnSpacings->0
				],
				ButtonBoxOptions->
					{ButtonFunction:>(SetParentCompoundButton[#]&),
					 ButtonEvaluator->Automatic,
					 Active->True}
		]],
		NotebookDefault
	];
	
ParentCompoundSectionCell[n_Integer]:=
	Cell[
		"with "<>ToString[n]<>" substituent site"<>If[n===1,":","s:"], 
		NotebookDefault,
		CellMargins->{{3, 0}, {0, 10}},
    	CellGroupingRules->{"SectionGrouping", 20},
		PageBreakAbove->True,
		LineSpacing->{1,3},
		FontSlant->"Italic"
	];

ParentCompoundDatabaseCell:=
	Map[
		Cell[CellGroupData[
			{ParentCompoundSectionCell[NumberOfSubstituentSites[First[#]]],
				ParentCompoundButtonCell[#]},
			Closed
		]]&,
		SortedParentCompounds
	];

ResetParentCompoundDatabase :=
	With[{cn = SelectedNotebook[]},
	 NotebookFind[idp,"ParentCompoundDatabaseSelection",All,CellTags];
	 SelectionMove[idp,All,CellGroup];
	 NotebookWrite[
		idp,
		Cell[
			CellGroupData[
				Prepend[
					ParentCompoundDatabaseCell,
					Cell[
	"To select a parent compound, click a molecule button below.",
						NotebookDefault,FontFamily -> "Helvetica", 
						FontSize -> 10,Selectable -> False,
						ShowGroupOpenCloseIcon -> False,
						CellTags-> "ParentCompoundDatabaseSelection"
					]
				],
				Open]
			],
			All
		];
	 SelectionMove[idp,Before,Notebook];
	 SetSelectedNotebook[cn]];

SetParentCompoundButton[mol_] :=
	With[{cn = SelectedNotebook[]},
	 ParentCompound=ToExpression[ToString[mol]];
	 NotebookFind[idp,"ParentCompoundSelection",All,CellTags];
	 NotebookWrite[
		idp,
		Cell[
			BoxData[
				RowBox[{
					StyleBox["The selected parent compound is:\n",
                  		FontFamily->"Helvetica", FontSize -> 10],
                	ButtonBox[
                		ToBoxes[ParentCompound],
                    	ButtonFunction:>(
                       		FrontEndExecute[
                        		{FrontEnd`NotebookApply[
                        			FrontEnd`InputNotebook[ ], #, All]}]&)
                        ],
                        "."
                    }]
				], 
				NotebookDefault,
				CellMargins->{{3, 3}, {7, 3}},
            	CellTags->"ParentCompoundSelection"
			],
			All
		];
	 SelectionMove[idp,Before,Notebook];
	 SetSelectedNotebook[cn]];



(* Parent Compound Input Panel *)


GetIsomersInputPanel := 
	(snb = SelectedNotebook[];
	 iip = NotebookOpen[IsomersInputPanel]);
	 
DiscardIsomersInputPanel := 
	(SetSelectedNotebook[snb];
	 NotebookClose[iip]);

IsomersInputPanel::nosites=
  "The number of substituent sites has not been entered!";

InsertRotation := 
  With[
	{n = (NotebookFind[iip,"NumberOfSubstituentSites",All,CellTags,
            AutoScroll->False];
		 ToExpression[First[NotebookRead[iip]]])},
	If[
		And[IntegerQ[n],Positive[n]],
		NotebookFind[iip,"RotationGenerators",
			All,CellTags,AutoScroll->False];
		If[
			First[NotebookRead[iip]]===BoxData[""],
			SelectionMove[iip,After,CellContents],
        	SelectionMove[iip,After,CellContents];
			NotebookWrite[iip,RowBox[{","}]]
		];
		NotebookWrite[
			iip,
			RowBox[{"(", GridBox[{
				Table[ToString[i],{i,1,n}],
				Table["\[Placeholder]",{n}]}
			],")"}],
			Placeholder
		],
		Message[IsomersInputPanel::nosites]
	]
  ];

InsertReflectionRotation := 
  With[
	{n = (NotebookFind[iip,"NumberOfSubstituentSites",All,CellTags,
            AutoScroll->False];
		 ToExpression[First[NotebookRead[iip]]])},
	If[
		And[IntegerQ[n],Positive[n]],
		NotebookFind[iip,"RotationReflectionGenerators",
			All,CellTags,AutoScroll->False];
		If[
			First[NotebookRead[iip]]===BoxData[""],
			SelectionMove[iip,After,CellContents],
        	SelectionMove[iip,After,CellContents];
			NotebookWrite[iip,RowBox[{","}]]
		];
		NotebookWrite[
			iip,
			SubscriptBox[RowBox[{"(", GridBox[{
				Table[ToString[i],{i,1,n}],
				Table["\[Placeholder]",{n}]}
			],")"}],"\[Sigma]"],
			Placeholder
		],
		Message[IsomersInputPanel::nosites]
	]
  ];
  
  InsertConformation := 
  With[
	{n = (NotebookFind[iip,"NumberOfSubstituentSites",All,CellTags,
            AutoScroll->False];
		 ToExpression[First[NotebookRead[iip]]])},
	If[
		And[IntegerQ[n],Positive[n]],
		NotebookFind[iip,"ConfirmationGenerators",
			All,CellTags,AutoScroll->False];
		If[
			First[NotebookRead[iip]]===BoxData[""],
			SelectionMove[iip,After,CellContents],
        	SelectionMove[iip,After,CellContents];
			NotebookWrite[iip,RowBox[{","}]]
		];
		NotebookWrite[
			iip,
			SubscriptBox[RowBox[{"(", GridBox[{
				Table[ToString[i],{i,1,n}],
				Table["\[Placeholder]",{n}]}
			],")"}],"c"],
			Placeholder
		],
		Message[IsomersInputPanel::nosites]
	]
  ];
  
  InsertConformationReflection := 
  With[
	{n = (NotebookFind[iip,"NumberOfSubstituentSites",All,CellTags,
            AutoScroll->False];
		 ToExpression[First[NotebookRead[iip]]])},
	If[
		And[IntegerQ[n],Positive[n]],
		NotebookFind[iip,"ConfirmationGenerators",
			All,CellTags,AutoScroll->False];
		If[
			First[NotebookRead[iip]]===BoxData[""],
			SelectionMove[iip,After,CellContents],
        	SelectionMove[iip,After,CellContents];
			NotebookWrite[iip,RowBox[{","}]]
		];
		NotebookWrite[
			iip,
			SubscriptBox[RowBox[{"(", GridBox[{
				Table[ToString[i],{i,1,n}],
				Table["\[Placeholder]",{n}]}
			],")"}],"c\[Sigma]"],
			Placeholder
		],
		Message[IsomersInputPanel::nosites]
	]
  ];
  
PasteParentCompoundDefinition :=
	RowBox[{"DefineParentCompound", "[", RowBox[{
		ToBoxes[NotebookFind[iip,"ParentCompoundSymbol",
			All,CellTags,AutoScroll->False];
		 ToExpression[First[NotebookRead[iip]]]],
		",",
		If[#==={},
			ToBoxes[NotebookFind[iip,"NumberOfSubstituentSites",
				All,CellTags,AutoScroll->False];
			ToExpression[First[NotebookRead[iip]]]],
 		    ToBoxes[#]
 		]&
		[Join[
			NotebookFind[iip,"RotationGenerators",
				All,CellTags,AutoScroll->False];
			ToExpression[Apply[RowBox[{"{",#,"}"}]&,
 		       	First[NotebookRead[iip]]]],
			NotebookFind[iip,"RotationReflectionGenerators",
   		 		All,CellTags,AutoScroll->False];
			ToExpression[Apply[RowBox[{"{",#,"}"}]&,
				First[NotebookRead[iip]]]],
			NotebookFind[iip,"ConfirmationGenerators",
				All,CellTags,AutoScroll->False];
			ToExpression[Apply[RowBox[{"{",#,"}"}]&,
   		     	First[NotebookRead[iip]]]]
   		]]}],
	"]"}]


ActivateParentCompoundInPanel :=
	DefineParentCompound[
		NotebookFind[iip,"ParentCompoundSymbol",
			All,CellTags,AutoScroll->False];
		 ToExpression[First[NotebookRead[iip]]],
		If[#==={},
			NotebookFind[iip,"NumberOfSubstituentSites",
				All,CellTags,AutoScroll->False];
			ToExpression[First[NotebookRead[iip]]],
 		    #
 		]&
		[Join[
			NotebookFind[iip,"RotationGenerators",
				All,CellTags,AutoScroll->False];
			ToExpression[Apply[RowBox[{"{",#,"}"}]&,
 		       	First[NotebookRead[iip]]]],
			NotebookFind[iip,"RotationReflectionGenerators",
   		 		All,CellTags,AutoScroll->False];
			ToExpression[Apply[RowBox[{"{",#,"}"}]&,
				First[NotebookRead[iip]]]],
			NotebookFind[iip,"ConfirmationGenerators",
				All,CellTags,AutoScroll->False];
			ToExpression[Apply[RowBox[{"{",#,"}"}]&,
   		     	First[NotebookRead[iip]]]]
   		]]
	];
	
	
(* Package Ending *)

End[]

Protect[
  ParentCompounds,
  ParentCompoundQ,
  ChiralParentCompoundQ,
  DefineParentCompound,
  DeleteParentCompound,
  GetParentCompounds,
  SaveParentCompounds,
  DefaultIsomersDatabase,
  IsomersControlPanel,
  IsomersDatabasePanel,
  ParentCompoundsInDataBase,
  NumberOfSubstituentSites,
  PointGroupGenerators,
  PointGroupElement,
  PointGroupElementQ,
  GeneratePointGroupElements,
  PointGroupCycleStructure,
  PointGroupCycle,
  PointGroup,
  PointGroupReport,
  CycleIndex,
  CycleIndices,
  PolyaPolynomial,
  PolyaVariable,
  NumberOfIsomers,
  NumberOfEnantiomericPairs,
  NumberOfAchiralIsomers,
  NumberOfDiastereoisomers,
  Conformations,Reflections,
  SubstituentsMultiplicity,
  Partitions,
  LimitedPartitions,
  InversePermutation,
  FromCycles,ToCycles,CyclesQ,
  PermutationQ
];

GetParentCompounds[];

EndPackage[]












